@@ -14,13 +14,18 @@
 * limitations under the License.
 *******************************************************************************/
 
-#ifndef C_TYPES_MAP_HPP
-#define C_TYPES_MAP_HPP
+#ifndef COMMON_C_TYPES_MAP_HPP
+#define COMMON_C_TYPES_MAP_HPP
 
 #include "dnnl_types.h"
 #include "gemm_types.hpp"
 #include "internal_desc_types.hpp"
 
+// These aliases should be in the global namespace as they are intended
+// to give names that better reflects the meaning of the entities
+using primitive_iface_t = dnnl_primitive;
+using primitive_desc_iface_t = dnnl_primitive_desc;
+
 namespace dnnl {
 namespace impl {
 
@@ -91,6 +96,7 @@ const alg_kind_t eltwise_sqrt_use_dst_for_bwd
 const alg_kind_t eltwise_logistic_use_dst_for_bwd
         = dnnl_eltwise_logistic_use_dst_for_bwd;
 const alg_kind_t eltwise_exp_use_dst_for_bwd = dnnl_eltwise_exp_use_dst_for_bwd;
+const alg_kind_t eltwise_round = dnnl_eltwise_round;
 const alg_kind_t pooling_max = dnnl_pooling_max;
 const alg_kind_t pooling_avg = dnnl_pooling_avg;
 const alg_kind_t pooling_avg_include_padding = dnnl_pooling_avg_include_padding;
@@ -165,6 +171,7 @@ const format_tag_t bacd = dnnl_bacd;
 const format_tag_t bca = dnnl_bca;
 const format_tag_t bcda = dnnl_bcda;
 const format_tag_t bcdea = dnnl_bcdea;
+const format_tag_t bacde = dnnl_bacde;
 const format_tag_t cba = dnnl_cba;
 const format_tag_t cdba = dnnl_cdba;
 const format_tag_t dcab = dnnl_dcab;
@@ -175,37 +182,48 @@ const format_tag_t Abc16a = dnnl_Abc16a;
 const format_tag_t ABc16a16b = dnnl_ABc16a16b;
 const format_tag_t ABc4a4b = dnnl_ABc4a4b;
 const format_tag_t aBc16b = dnnl_aBc16b;
+const format_tag_t aBc32b = dnnl_aBc32b;
 const format_tag_t ABc16b16a = dnnl_ABc16b16a;
 const format_tag_t Abc4a = dnnl_Abc4a;
 const format_tag_t aBc4b = dnnl_aBc4b;
 const format_tag_t ABc4b16a4b = dnnl_ABc4b16a4b;
 const format_tag_t ABc2b8a4b = dnnl_ABc2b8a4b;
+const format_tag_t ABc16b16a4b = dnnl_ABc16b16a4b;
+const format_tag_t ABc16b16a2b = dnnl_ABc16b16a2b;
 const format_tag_t ABc4b4a = dnnl_ABc4b4a;
 const format_tag_t ABc8a16b2a = dnnl_ABc8a16b2a;
 const format_tag_t BAc8a16b2a = dnnl_BAc8a16b2a;
 const format_tag_t ABc8a8b = dnnl_ABc8a8b;
+const format_tag_t ABc8a4b = dnnl_ABc8a4b;
 const format_tag_t aBc8b = dnnl_aBc8b;
 const format_tag_t ABc8b16a2b = dnnl_ABc8b16a2b;
 const format_tag_t ABc8b8a = dnnl_ABc8b8a;
 const format_tag_t Abcd16a = dnnl_Abcd16a;
 const format_tag_t Abcd8a = dnnl_Abcd8a;
+const format_tag_t Abcd32a = dnnl_Abcd32a;
 const format_tag_t ABcd16a16b = dnnl_ABcd16a16b;
 const format_tag_t aBcd16b = dnnl_aBcd16b;
+const format_tag_t aBcd32b = dnnl_aBcd32b;
 const format_tag_t ABcd16b16a = dnnl_ABcd16b16a;
 const format_tag_t aBCd16b16c = dnnl_aBCd16b16c;
 const format_tag_t aBCd16c16b = dnnl_aBCd16c16b;
 const format_tag_t Abcd4a = dnnl_Abcd4a;
 const format_tag_t aBcd4b = dnnl_aBcd4b;
 const format_tag_t ABcd4b16a4b = dnnl_ABcd4b16a4b;
+const format_tag_t ABcd16b16a4b = dnnl_ABcd16b16a4b;
+const format_tag_t ABcd16b16a2b = dnnl_ABcd16b16a2b;
 const format_tag_t ABcd4b4a = dnnl_ABcd4b4a;
 const format_tag_t ABcd4a4b = dnnl_ABcd4a4b;
 const format_tag_t aBCd4c16b4c = dnnl_aBCd4c16b4c;
 const format_tag_t aBCd2c8b4c = dnnl_aBCd2c8b4c;
+const format_tag_t aBCd16c16b4c = dnnl_aBCd16c16b4c;
+const format_tag_t aBCd16c16b2c = dnnl_aBCd16c16b2c;
 const format_tag_t aBCd4c4b = dnnl_aBCd4c4b;
 const format_tag_t aBCd4b4c = dnnl_aBCd4b4c;
 const format_tag_t ABcd8a16b2a = dnnl_ABcd8a16b2a;
 const format_tag_t BAcd8a16b2a = dnnl_BAcd8a16b2a;
 const format_tag_t ABcd8a8b = dnnl_ABcd8a8b;
+const format_tag_t ABcd8a4b = dnnl_ABcd8a4b;
 const format_tag_t aBcd8b = dnnl_aBcd8b;
 const format_tag_t ABcd8b16a2b = dnnl_ABcd8b16a2b;
 const format_tag_t ABcd2b8a4b = dnnl_ABcd2b8a4b;
@@ -213,11 +231,14 @@ const format_tag_t aBCd8b16c2b = dnnl_aBCd8b16c2b;
 const format_tag_t aCBd8b16c2b = dnnl_aCBd8b16c2b;
 const format_tag_t ABcd8b8a = dnnl_ABcd8b8a;
 const format_tag_t aBCd8b8c = dnnl_aBCd8b8c;
+const format_tag_t aBCd8b4c = dnnl_aBCd8b4c;
 const format_tag_t aBCd8c16b2c = dnnl_aBCd8c16b2c;
 const format_tag_t aBCd8c8b = dnnl_aBCd8c8b;
 const format_tag_t Abcde16a = dnnl_Abcde16a;
+const format_tag_t Abcde32a = dnnl_Abcde32a;
 const format_tag_t ABcde16a16b = dnnl_ABcde16a16b;
 const format_tag_t aBcde16b = dnnl_aBcde16b;
+const format_tag_t aBcde32b = dnnl_aBcde32b;
 const format_tag_t ABcde16b16a = dnnl_ABcde16b16a;
 const format_tag_t aBCde16b16c = dnnl_aBCde16b16c;
 const format_tag_t aBCde16c16b = dnnl_aBCde16c16b;
@@ -228,9 +249,12 @@ const format_tag_t ABcde4b4a = dnnl_ABcde4b4a;
 const format_tag_t ABcde4a4b = dnnl_ABcde4a4b;
 const format_tag_t aBCde4b4c = dnnl_aBCde4b4c;
 const format_tag_t aBCde4c16b4c = dnnl_aBCde4c16b4c;
+const format_tag_t aBCde16c16b4c = dnnl_aBCde16c16b4c;
+const format_tag_t aBCde16c16b2c = dnnl_aBCde16c16b2c;
 const format_tag_t aBCde4c4b = dnnl_aBCde4c4b;
 const format_tag_t Abcde8a = dnnl_Abcde8a;
 const format_tag_t ABcde8a8b = dnnl_ABcde8a8b;
+const format_tag_t ABcde8a4b = dnnl_ABcde8a4b;
 const format_tag_t aBcde8b = dnnl_aBcde8b;
 const format_tag_t ABcde8b16a2b = dnnl_ABcde8b16a2b;
 const format_tag_t ABcde8a16b2a = dnnl_ABcde8a16b2a;
@@ -241,9 +265,20 @@ const format_tag_t aBCde8b16c2b = dnnl_aBCde8b16c2b;
 const format_tag_t aCBde8b16c2b = dnnl_aCBde8b16c2b;
 const format_tag_t ABcde8b8a = dnnl_ABcde8b8a;
 const format_tag_t aBCde8b8c = dnnl_aBCde8b8c;
+const format_tag_t aBCde8b4c = dnnl_aBCde8b4c;
+const format_tag_t ABc4a8b8a4b = dnnl_ABc4a8b8a4b;
 const format_tag_t ABcd4a8b8a4b = dnnl_ABcd4a8b8a4b;
+const format_tag_t ABcde4a8b8a4b = dnnl_ABcde4a8b8a4b;
 const format_tag_t ABcd2a8b8a2b = dnnl_ABcd2a8b8a2b;
+const format_tag_t aBCd4b8c8b4c = dnnl_aBCd4b8c8b4c;
 const format_tag_t aBCde4b8c8b4c = dnnl_aBCde4b8c8b4c;
+const format_tag_t aBCdef4b8c8b4c = dnnl_aBCdef4b8c8b4c;
+const format_tag_t BAc4b8a8b4a = dnnl_BAc4b8a8b4a;
+const format_tag_t BAcd4b8a8b4a = dnnl_BAcd4b8a8b4a;
+const format_tag_t BAcde4b8a8b4a = dnnl_BAcde4b8a8b4a;
+const format_tag_t aCBd4c8b8c4b = dnnl_aCBd4c8b8c4b;
+const format_tag_t aCBde4c8b8c4b = dnnl_aCBde4c8b8c4b;
+const format_tag_t aCBdef4c8b8c4b = dnnl_aCBdef4c8b8c4b;
 const format_tag_t aBCde2b8c8b2c = dnnl_aBCde2b8c8b2c;
 const format_tag_t aBCde8c16b2c = dnnl_aBCde8c16b2c;
 const format_tag_t aBCde8c8b = dnnl_aBCde8c8b;
@@ -256,16 +291,19 @@ const format_tag_t aBcdef4b = dnnl_aBcdef4b;
 const format_tag_t aBCdef4c4b = dnnl_aBCdef4c4b;
 const format_tag_t aBCdef4b4c = dnnl_aBCdef4b4c;
 const format_tag_t aBCdef8b8c = dnnl_aBCdef8b8c;
+const format_tag_t aBCdef8b4c = dnnl_aBCdef8b4c;
 const format_tag_t aBCdef8c16b2c = dnnl_aBCdef8c16b2c;
 const format_tag_t aBCdef8b16c2b = dnnl_aBCdef8b16c2b;
 const format_tag_t aCBdef8b16c2b = dnnl_aCBdef8b16c2b;
 const format_tag_t aBCdef8c8b = dnnl_aBCdef8c8b;
 const format_tag_t aBdc16b = dnnl_aBdc16b;
 const format_tag_t aBdC16b2c = dnnl_aBdC16b2c;
+const format_tag_t aBdC16b4c = dnnl_aBdC16b4c;
 const format_tag_t aBdc4b = dnnl_aBdc4b;
 const format_tag_t aBdc8b = dnnl_aBdc8b;
 const format_tag_t aBdec16b = dnnl_aBdec16b;
 const format_tag_t aBdeC16b2c = dnnl_aBdeC16b2c;
+const format_tag_t aBdeC16b4c = dnnl_aBdeC16b4c;
 const format_tag_t aBdec4b = dnnl_aBdec4b;
 const format_tag_t aBdec8b = dnnl_aBdec8b;
 const format_tag_t aBdefc16b = dnnl_aBdefc16b;
@@ -276,6 +314,7 @@ const format_tag_t aBdefc4b = dnnl_aBdefc4b;
 const format_tag_t aBdefc8b = dnnl_aBdefc8b;
 const format_tag_t Acb16a = dnnl_Acb16a;
 const format_tag_t AcB16a2b = dnnl_AcB16a2b;
+const format_tag_t AcB16a4b = dnnl_AcB16a4b;
 const format_tag_t Acb4a = dnnl_Acb4a;
 const format_tag_t Acb8a = dnnl_Acb8a;
 const format_tag_t aCBd16b16c = dnnl_aCBd16b16c;
@@ -284,6 +323,7 @@ const format_tag_t aCBde16b16c = dnnl_aCBde16b16c;
 const format_tag_t aCBde16c16b = dnnl_aCBde16c16b;
 const format_tag_t Acdb16a = dnnl_Acdb16a;
 const format_tag_t AcdB16a2b = dnnl_AcdB16a2b;
+const format_tag_t AcdB16a4b = dnnl_AcdB16a4b;
 const format_tag_t Acdb4a = dnnl_Acdb4a;
 const format_tag_t Acdb8a = dnnl_Acdb8a;
 const format_tag_t Acdeb16a = dnnl_Acdeb16a;
@@ -292,11 +332,14 @@ const format_tag_t Acdeb4a = dnnl_Acdeb4a;
 const format_tag_t Acdeb8a = dnnl_Acdeb8a;
 const format_tag_t BAc16a16b = dnnl_BAc16a16b;
 const format_tag_t BAcd16a16b = dnnl_BAcd16a16b;
+const format_tag_t ABc32a32b = dnnl_ABc32a32b;
 const format_tag_t BAcde16a16b = dnnl_BAcde16a16b;
 const format_tag_t ABcd32a32b = dnnl_ABcd32a32b;
+const format_tag_t ABcde32a32b = dnnl_ABcde32a32b;
 const format_tag_t BAcde16b16a = dnnl_BAcde16b16a;
 const format_tag_t aBdec32b = dnnl_aBdec32b;
 const format_tag_t Abcdef16a = dnnl_Abcdef16a;
+const format_tag_t Abcdef32a = dnnl_Abcdef32a;
 const format_tag_t Acdb32a = dnnl_Acdb32a;
 const format_tag_t BAc16b16a = dnnl_BAc16b16a;
 const format_tag_t BAcd16b16a = dnnl_BAcd16b16a;
@@ -339,6 +382,7 @@ const format_tag_t oidhw = dnnl_oidhw;
 const format_tag_t dhwio = dnnl_dhwio;
 const format_tag_t odhwi = dnnl_odhwi;
 const format_tag_t idhwo = dnnl_idhwo;
+const format_tag_t iodhw = dnnl_iodhw;
 const format_tag_t goiw = dnnl_goiw;
 const format_tag_t goihw = dnnl_goihw;
 const format_tag_t wigo = dnnl_wigo;
@@ -355,19 +399,24 @@ const format_tag_t ldgoi = dnnl_ldgoi;
 const format_tag_t ldio = dnnl_ldio;
 const format_tag_t ldoi = dnnl_ldoi;
 const format_tag_t ldgo = dnnl_ldgo;
+const format_tag_t nCdhw32c = dnnl_nCdhw32c;
 const format_tag_t nCdhw16c = dnnl_nCdhw16c;
 const format_tag_t nCdhw4c = dnnl_nCdhw4c;
 const format_tag_t nCdhw8c = dnnl_nCdhw8c;
+const format_tag_t nChw32c = dnnl_nChw32c;
 const format_tag_t nChw16c = dnnl_nChw16c;
 const format_tag_t nChw4c = dnnl_nChw4c;
 const format_tag_t nChw8c = dnnl_nChw8c;
+const format_tag_t nCw32c = dnnl_nCw32c;
 const format_tag_t nCw16c = dnnl_nCw16c;
 const format_tag_t nCw4c = dnnl_nCw4c;
 const format_tag_t nCw8c = dnnl_nCw8c;
 const format_tag_t NCw16n16c = dnnl_NCw16n16c;
 const format_tag_t NChw16n16c = dnnl_NChw16n16c;
 const format_tag_t NCdhw16n16c = dnnl_NCdhw16n16c;
+const format_tag_t NCw32n32c = dnnl_NCw32n32c;
 const format_tag_t NChw32n32c = dnnl_NChw32n32c;
+const format_tag_t NCdhw32n32c = dnnl_NCdhw32n32c;
 const format_tag_t IOdhw16i16o = dnnl_IOdhw16i16o;
 const format_tag_t IOhw16i16o = dnnl_IOhw16i16o;
 const format_tag_t Ohwi32o = dnnl_Ohwi32o;
@@ -382,6 +431,8 @@ const format_tag_t OIw16o16i = dnnl_OIw16o16i;
 const format_tag_t Oiw16o = dnnl_Oiw16o;
 const format_tag_t OIw4i16o4i = dnnl_OIw4i16o4i;
 const format_tag_t OIw2i8o4i = dnnl_OIw2i8o4i;
+const format_tag_t OIw16i16o4i = dnnl_OIw16i16o4i;
+const format_tag_t OIw16i16o2i = dnnl_OIw16i16o2i;
 const format_tag_t OIw4i4o = dnnl_OIw4i4o;
 const format_tag_t OIw4o4i = dnnl_OIw4o4i;
 const format_tag_t Oiw4o = dnnl_Oiw4o;
@@ -390,20 +441,25 @@ const format_tag_t OIw8i8o = dnnl_OIw8i8o;
 const format_tag_t OIw8o16i2o = dnnl_OIw8o16i2o;
 const format_tag_t IOw8o16i2o = dnnl_IOw8o16i2o;
 const format_tag_t OIw8o8i = dnnl_OIw8o8i;
+const format_tag_t OIw8o4i = dnnl_OIw8o4i;
 const format_tag_t Owi16o = dnnl_Owi16o;
 const format_tag_t OwI16o2i = dnnl_OwI16o2i;
+const format_tag_t OwI16o4i = dnnl_OwI16o4i;
 const format_tag_t Owi4o = dnnl_Owi4o;
 const format_tag_t Owi8o = dnnl_Owi8o;
 const format_tag_t IOdhw16o16i = dnnl_IOdhw16o16i;
 const format_tag_t IOhw16o16i = dnnl_IOhw16o16i;
 const format_tag_t Ohwi16o = dnnl_Ohwi16o;
 const format_tag_t OhwI16o2i = dnnl_OhwI16o2i;
+const format_tag_t OhwI16o4i = dnnl_OhwI16o4i;
 const format_tag_t Ohwi4o = dnnl_Ohwi4o;
 const format_tag_t Ohwi8o = dnnl_Ohwi8o;
 const format_tag_t OIhw16i16o = dnnl_OIhw16i16o;
 const format_tag_t OIhw16o16i = dnnl_OIhw16o16i;
 const format_tag_t Oihw16o = dnnl_Oihw16o;
 const format_tag_t OIhw4i16o4i = dnnl_OIhw4i16o4i;
+const format_tag_t OIhw16i16o4i = dnnl_OIhw16i16o4i;
+const format_tag_t OIhw16i16o2i = dnnl_OIhw16i16o2i;
 const format_tag_t OIhw4i4o = dnnl_OIhw4i4o;
 const format_tag_t OIhw4o4i = dnnl_OIhw4o4i;
 const format_tag_t Oihw4o = dnnl_Oihw4o;
@@ -413,6 +469,7 @@ const format_tag_t OIhw8i8o = dnnl_OIhw8i8o;
 const format_tag_t OIhw8o16i2o = dnnl_OIhw8o16i2o;
 const format_tag_t IOhw8o16i2o = dnnl_IOhw8o16i2o;
 const format_tag_t OIhw8o8i = dnnl_OIhw8o8i;
+const format_tag_t OIhw8o4i = dnnl_OIhw8o4i;
 const format_tag_t Odhwi16o = dnnl_Odhwi16o;
 const format_tag_t OdhwI16o2i = dnnl_OdhwI16o2i;
 const format_tag_t Odhwi4o = dnnl_Odhwi4o;
@@ -430,6 +487,7 @@ const format_tag_t OIdhw8o16i2o = dnnl_OIdhw8o16i2o;
 const format_tag_t IOdhw8o16i2o = dnnl_IOdhw8o16i2o;
 const format_tag_t OIdhw8i8o = dnnl_OIdhw8i8o;
 const format_tag_t OIdhw8o8i = dnnl_OIdhw8o8i;
+const format_tag_t OIdhw8o4i = dnnl_OIdhw8o4i;
 const format_tag_t gIOw16o16i = dnnl_gIOw16o16i;
 const format_tag_t Goiw16g = dnnl_Goiw16g;
 const format_tag_t Goiw8g = dnnl_Goiw8g;
@@ -438,6 +496,8 @@ const format_tag_t gOIw16o16i = dnnl_gOIw16o16i;
 const format_tag_t gOiw16o = dnnl_gOiw16o;
 const format_tag_t gOIw4i16o4i = dnnl_gOIw4i16o4i;
 const format_tag_t gOIw2i8o4i = dnnl_gOIw2i8o4i;
+const format_tag_t gOIw16i16o4i = dnnl_gOIw16i16o4i;
+const format_tag_t gOIw16i16o2i = dnnl_gOIw16i16o2i;
 const format_tag_t gOIw4i4o = dnnl_gOIw4i4o;
 const format_tag_t gOIw4o4i = dnnl_gOIw4o4i;
 const format_tag_t gOiw4o = dnnl_gOiw4o;
@@ -446,14 +506,17 @@ const format_tag_t gOIw8i8o = dnnl_gOIw8i8o;
 const format_tag_t gOIw8o16i2o = dnnl_gOIw8o16i2o;
 const format_tag_t gIOw8o16i2o = dnnl_gIOw8o16i2o;
 const format_tag_t gOIw8o8i = dnnl_gOIw8o8i;
+const format_tag_t gOIw8o4i = dnnl_gOIw8o4i;
 const format_tag_t gOwi16o = dnnl_gOwi16o;
 const format_tag_t gOwI16o2i = dnnl_gOwI16o2i;
+const format_tag_t gOwI16o4i = dnnl_gOwI16o4i;
 const format_tag_t gOwi4o = dnnl_gOwi4o;
 const format_tag_t gOwi8o = dnnl_gOwi8o;
 const format_tag_t gIOdhw16o16i = dnnl_gIOdhw16o16i;
 const format_tag_t gIOhw16o16i = dnnl_gIOhw16o16i;
 const format_tag_t gOhwi16o = dnnl_gOhwi16o;
 const format_tag_t gOhwI16o2i = dnnl_gOhwI16o2i;
+const format_tag_t gOhwI16o4i = dnnl_gOhwI16o4i;
 const format_tag_t gOhwi4o = dnnl_gOhwi4o;
 const format_tag_t gOhwi8o = dnnl_gOhwi8o;
 const format_tag_t Goihw16g = dnnl_Goihw16g;
@@ -462,6 +525,8 @@ const format_tag_t gOIhw16o16i = dnnl_gOIhw16o16i;
 const format_tag_t gOihw16o = dnnl_gOihw16o;
 const format_tag_t gOIhw2i8o4i = dnnl_gOIhw2i8o4i;
 const format_tag_t gOIhw4i16o4i = dnnl_gOIhw4i16o4i;
+const format_tag_t gOIhw16i16o4i = dnnl_gOIhw16i16o4i;
+const format_tag_t gOIhw16i16o2i = dnnl_gOIhw16i16o2i;
 const format_tag_t gOIhw4i4o = dnnl_gOIhw4i4o;
 const format_tag_t gOIhw4o4i = dnnl_gOIhw4o4i;
 const format_tag_t gOihw4o = dnnl_gOihw4o;
@@ -469,12 +534,23 @@ const format_tag_t Goihw8g = dnnl_Goihw8g;
 const format_tag_t gOIhw8i16o2i = dnnl_gOIhw8i16o2i;
 const format_tag_t gOIhw8i8o = dnnl_gOIhw8i8o;
 const format_tag_t gOIhw8o16i2o = dnnl_gOIhw8o16i2o;
+const format_tag_t OIw4o8i8o4i = dnnl_OIw4o8i8o4i;
 const format_tag_t gIOhw8o16i2o = dnnl_gIOhw8o16i2o;
 const format_tag_t OIhw4o8i8o4i = dnnl_OIhw4o8i8o4i;
+const format_tag_t OIdhw4o8i8o4i = dnnl_OIdhw4o8i8o4i;
+const format_tag_t IOw4i8o8i4o = dnnl_IOw4i8o8i4o;
+const format_tag_t IOhw4i8o8i4o = dnnl_IOhw4i8o8i4o;
+const format_tag_t IOdhw4i8o8i4o = dnnl_IOdhw4i8o8i4o;
+const format_tag_t gIOw4i8o8i4o = dnnl_gIOw4i8o8i4o;
+const format_tag_t gIOhw4i8o8i4o = dnnl_gIOhw4i8o8i4o;
+const format_tag_t gIOdhw4i8o8i4o = dnnl_gIOdhw4i8o8i4o;
 const format_tag_t OIhw2o8i8o2i = dnnl_OIhw2o8i8o2i;
+const format_tag_t gOIw4o8i8o4i = dnnl_gOIw4o8i8o4i;
 const format_tag_t gOIhw4o8i8o4i = dnnl_gOIhw4o8i8o4i;
+const format_tag_t gOIdhw4o8i8o4i = dnnl_gOIdhw4o8i8o4i;
 const format_tag_t gOIhw2o8i8o2i = dnnl_gOIhw2o8i8o2i;
 const format_tag_t gOIhw8o8i = dnnl_gOIhw8o8i;
+const format_tag_t gOIhw8o4i = dnnl_gOIhw8o4i;
 const format_tag_t gIOdhw16i16o = dnnl_gIOdhw16i16o;
 const format_tag_t gOdhwi16o = dnnl_gOdhwi16o;
 const format_tag_t gOdhwI16o2i = dnnl_gOdhwI16o2i;
@@ -493,6 +569,10 @@ const format_tag_t gOIdhw8o16i2o = dnnl_gOIdhw8o16i2o;
 const format_tag_t gIOdhw8o16i2o = dnnl_gIOdhw8o16i2o;
 const format_tag_t gOIdhw8i8o = dnnl_gOIdhw8i8o;
 const format_tag_t gOIdhw8o8i = dnnl_gOIdhw8o8i;
+const format_tag_t gOIdhw8o4i = dnnl_gOIdhw8o4i;
+const format_tag_t Goiw32g = dnnl_Goiw32g;
+const format_tag_t Goihw32g = dnnl_Goihw32g;
+const format_tag_t Goidhw32g = dnnl_Goidhw32g;
 const format_tag_t gOIw2i4o2i = dnnl_gOIw2i4o2i;
 const format_tag_t gOIhw2i4o2i = dnnl_gOIhw2i4o2i;
 const format_tag_t gOIdhw2i4o2i = dnnl_gOIdhw2i4o2i;
@@ -564,6 +644,10 @@ const primitive_kind_t binary = dnnl_binary;
 const primitive_kind_t logsoftmax = dnnl_logsoftmax;
 const primitive_kind_t matmul = dnnl_matmul;
 const primitive_kind_t resampling = dnnl_resampling;
+
+// Internal only primitive kinds.
+const primitive_kind_t internal_only_start = (primitive_kind_t)(1 << 12);
+const primitive_kind_t zero_pad = internal_only_start;
 } // namespace primitive_kind
 
 using query_t = dnnl_query_t;
@@ -618,6 +702,10 @@ const query_t exec_arg_md = dnnl_query_exec_arg_md;
 
 const query_t workspace_md = dnnl_query_workspace_md;
 const query_t scratchpad_md = dnnl_query_scratchpad_md;
+
+// Internal only query kinds.
+const query_t internal_only_start = (query_t)(1 << 12);
+const query_t zero_pad_d = internal_only_start;
 } // namespace query
 
 using blocking_desc_t = dnnl_blocking_desc_t;
@@ -650,6 +738,7 @@ using gemm_desc_t = dnnl_gemm_desc_t;
 using concat_desc_t = dnnl_concat_desc_t;
 using reorder_desc_t = dnnl_reorder_desc_t;
 using sum_desc_t = dnnl_sum_desc_t;
+using zero_pad_desc_t = dnnl_zero_pad_desc_t;
 
 /* C op_desc_t, which eventually are just (void*) */
 using c_op_desc_t = dnnl_op_desc_t;
@@ -676,6 +765,7 @@ struct op_desc_t {
         binary_desc_t binary;
         matmul_desc_t matmul;
         resampling_desc_t resampling;
+        zero_pad_desc_t zero_pad;
     };
 
 #define DECL_CTOR_AND_CONVERTERS(c_type) \
@@ -704,6 +794,7 @@ struct op_desc_t {
     DECL_CTOR_AND_CONVERTERS(binary_desc_t);
     DECL_CTOR_AND_CONVERTERS(matmul_desc_t);
     DECL_CTOR_AND_CONVERTERS(resampling_desc_t);
+    DECL_CTOR_AND_CONVERTERS(zero_pad_desc_t);
 
     // concat_desc_t and sum_desc_t have data members which have non-trivial
     // special member functions hence the default destructor is implicitly
@@ -716,11 +807,9 @@ struct op_desc_t {
 
 using engine_t = dnnl_engine;
 using primitive_desc_iterator_t = dnnl_primitive_desc_iterator;
-using primitive_desc_t = dnnl_primitive_desc;
 using primitive_attr_t = dnnl_primitive_attr;
 using post_ops_t = dnnl_post_ops;
 using memory_t = dnnl_memory;
-using primitive_t = dnnl_primitive;
 
 using stream_flags_t = dnnl_stream_flags_t;
 namespace stream_flags {
