@@ -14,39 +14,32 @@
 * limitations under the License.
 *******************************************************************************/
 
-#ifndef REF_CONCAT_HPP
-#define REF_CONCAT_HPP
+#ifndef CPU_REF_CONCAT_HPP
+#define CPU_REF_CONCAT_HPP
 
-#include "engine.hpp"
-#include "reorder_pd.hpp"
+#include "common/engine.hpp"
+#include "common/primitive.hpp"
+#include "common/reorder_pd.hpp"
+#include "common/stream.hpp"
 
-#include "cpu_concat_pd.hpp"
+#include "cpu/cpu_concat_pd.hpp"
 
 namespace dnnl {
 namespace impl {
 namespace cpu {
 
-struct ref_concat_t : public primitive_impl_t {
+struct ref_concat_t : public primitive_t {
     struct pd_t : public cpu_concat_pd_t {
-        pd_t(engine_t *engine, const primitive_attr_t *attr,
-                const memory_desc_t *dst_md, int n, int concat_dim,
-                const memory_desc_t *src_mds)
-            : cpu_concat_pd_t(engine, attr, dst_md, n, concat_dim, src_mds)
+        pd_t(const primitive_attr_t *attr, const memory_desc_t *dst_md, int n,
+                int concat_dim, const memory_desc_t *src_mds)
+            : cpu_concat_pd_t(attr, dst_md, n, concat_dim, src_mds)
             , tent_dst_md_(types::zero_md()) {}
         pd_t(const pd_t &rhs) : cpu_concat_pd_t(rhs) { copy(rhs); }
-        ~pd_t() { clear(); }
-
-        pd_t &operator=(const pd_t &rhs) {
-            DNNL_SHORT_CIRCUIT_SELF_ASSIGN(rhs);
-            cpu_concat_pd_t::operator=(rhs);
-            clear();
-            copy(rhs);
-            return *this;
-        }
+        ~pd_t() = default;
 
         DECLARE_CONCAT_PD_T("ref:any", ref_concat_t);
 
-        status_t init() {
+        status_t init(engine_t *engine) {
             status_t status = cpu_concat_pd_t::init();
             if (status != status::success) {
                 assert(dst_md_.format_kind != format_kind::undef);
@@ -60,15 +53,17 @@ struct ref_concat_t : public primitive_impl_t {
             }
 
             for (int i = 0; i < n_; ++i) {
-                auto r_impls = engine_->get_reorder_implementation_list(
+                auto r_impls = engine->get_reorder_implementation_list(
                         src_md(i), src_image_md(i));
                 for (auto r = r_impls; *r; ++r) {
-                    const primitive_attr_t attr; /* alpha == 1. */
+                    primitive_attr_t r_attr; /* alpha == 1. */
+                    r_attr.set_scratchpad_mode(scratchpad_mode::user);
                     reorder_pd_t *r_pd = nullptr;
-                    if ((*r)(&r_pd, engine_, &attr, engine_, src_md(i), engine_,
+
+                    if ((*r)(&r_pd, engine, &r_attr, engine, src_md(i), engine,
                                 src_image_md(i))
                             == status::success) {
-                        reorder_pds_.push_back(r_pd);
+                        reorder_pds_.emplace_back(r_pd);
                         break;
                     }
                 }
@@ -79,65 +74,82 @@ struct ref_concat_t : public primitive_impl_t {
                 assert(tent_dst_md_.format_kind != format_kind::undef);
                 assert(dst_md_.format_kind != format_kind::undef);
 
-                primitive_desc_t *r_pd = nullptr;
-                status = dnnl_reorder_primitive_desc_create(&r_pd,
-                        &tent_dst_md_, engine_, &dst_md_, engine_, nullptr);
-                if (status != status::success) return status;
-                reorder_pds_.push_back((const reorder_pd_t *)r_pd);
-
-                using namespace memory_tracking::names;
-                auto scratchpad = scratchpad_registry().registrar();
-                scratchpad.book(memory_tracking::names::key_concat_tent_dst,
-                        dnnl_memory_desc_get_size(&tent_dst_md_));
+                auto r_impls = engine->get_reorder_implementation_list(
+                        &tent_dst_md_, &dst_md_);
+                for (auto r = r_impls; *r; ++r) {
+                    primitive_attr_t r_attr;
+                    r_attr.set_scratchpad_mode(scratchpad_mode::user);
+                    reorder_pd_t *r_pd = nullptr;
+                    if ((*r)(&r_pd, engine, &r_attr, engine, &tent_dst_md_,
+                                engine, &dst_md_)
+                            == status::success) {
+                        reorder_pds_.emplace_back(r_pd);
+                        break;
+                    }
+                }
             }
-
+            init_scratchpad();
             return status;
         }
 
         // if dst is forced and cannot be used directly.
         bool use_tent_dst() const { return !types::is_zero_md(&tent_dst_md_); }
 
-        std::vector<const reorder_pd_t *> reorder_pds_;
+        std::vector<std::unique_ptr<primitive_desc_t>> reorder_pds_;
         memory_desc_t tent_dst_md_;
 
     private:
         void copy(const pd_t &rhs) {
             tent_dst_md_ = rhs.tent_dst_md_;
             for (size_t i = 0; i < rhs.reorder_pds_.size(); ++i)
-                reorder_pds_.push_back(
-                        (const reorder_pd_t *)rhs.reorder_pds_[i]->clone());
+                reorder_pds_.emplace_back(rhs.reorder_pds_[i]->clone());
         }
 
-        void clear() {
-            for (auto &rpd : reorder_pds_)
-                delete rpd;
+        void init_scratchpad() {
+            using namespace memory_tracking::names;
+            auto scratchpad = scratchpad_registry().registrar();
+            if (use_tent_dst()) {
+                const memory_desc_wrapper tent_dst_d(&tent_dst_md_);
+                scratchpad.book(memory_tracking::names::key_concat_tent_dst,
+                        tent_dst_d.size(), 1, tent_dst_d.data_type_size());
+            }
+
+            for (size_t i = 0; i < reorder_pds_.size(); i++) {
+                scratchpad.book(key_nested_multiple + (int)i,
+                        reorder_pds_[i]->scratchpad_registry());
+            }
         }
     };
 
-    ref_concat_t(const pd_t *apd) : primitive_impl_t(apd) {
+    ref_concat_t(const pd_t *apd) : primitive_t(apd) {}
+
+    status_t init(engine_t *engine) override {
         const size_t n = pd()->reorder_pds_.size();
         reorders_.resize(n);
         for (size_t i = 0; i < n; ++i)
-            pd()->reorder_pds_[i]->create_primitive(&reorders_[i]);
+            pd()->reorder_pds_[i]->create_primitive(reorders_[i], engine);
+        return status::success;
     }
 
-    ~ref_concat_t() {
-        for (auto &r : reorders_)
-            delete r;
-    }
+    ~ref_concat_t() = default;
 
-    virtual status_t execute(const exec_ctx_t &ctx) const override {
+    status_t execute(const exec_ctx_t &ctx) const override {
+        using namespace memory_tracking::names;
+        engine_t *engine = ctx.stream()->engine();
         const auto n = pd()->n_inputs();
 
-        auto execute_reorder
-                = [&](const primitive_t *reorder, const memory_arg_t &src,
-                          const memory_arg_t &dst) {
-                      exec_args_t r_args;
-                      r_args[DNNL_ARG_SRC] = src;
-                      r_args[DNNL_ARG_DST] = dst;
-                      exec_ctx_t r_ctx(ctx.stream(), std::move(r_args));
-                      reorder->execute(r_ctx);
-                  };
+        auto execute_reorder = [&](const std::shared_ptr<primitive_t> &reorder,
+                                       const memory_arg_t &src,
+                                       const memory_arg_t &dst, int r_num) {
+            exec_args_t r_args;
+            r_args[DNNL_ARG_SRC] = src;
+            r_args[DNNL_ARG_DST] = dst;
+            exec_ctx_t r_ctx(ctx.stream(), std::move(r_args));
+
+            nested_scratchpad_t ns(ctx, key_nested_multiple + r_num, reorder);
+            r_ctx.set_scratchpad_grantor(ns.grantor());
+            reorder->execute(r_ctx);
+        };
 
         const unsigned submemory_flags = memory_flags_t::use_runtime_ptr
                 | memory_flags_t::omit_zero_pad;
@@ -148,34 +160,35 @@ struct ref_concat_t : public primitive_impl_t {
                     memory_tracking::names::key_concat_tent_dst);
 
             for (int i = 0; i < n; ++i) {
-                memory_t tent_dst_i(engine(), pd()->src_image_md(i),
+                memory_t tent_dst_i(engine, pd()->src_image_md(i),
                         submemory_flags, tent_dst_ptr);
                 execute_reorder(reorders_[i],
                         ctx.args().at(DNNL_ARG_MULTIPLE_SRC + i),
-                        {&tent_dst_i, false});
+                        {&tent_dst_i, false}, i);
             }
 
-            memory_t tent_dst(engine(), &pd()->tent_dst_md_,
+            memory_t tent_dst(engine, &pd()->tent_dst_md_,
                     memory_flags_t::use_runtime_ptr, tent_dst_ptr);
+
             execute_reorder(reorders_[n], {&tent_dst, true},
-                    ctx.args().at(DNNL_ARG_DST));
+                    ctx.args().at(DNNL_ARG_DST), n);
         } else {
             auto dst_ptr = CTX_OUT_MEM(void *, DNNL_ARG_DST);
             for (int i = 0; i < n; ++i) {
-                memory_t tent_dst_i(engine(), pd()->src_image_md(i),
+                memory_t tent_dst_i(engine, pd()->src_image_md(i),
                         submemory_flags, dst_ptr);
+
                 execute_reorder(reorders_[i],
                         ctx.args().at(DNNL_ARG_MULTIPLE_SRC + i),
-                        {&tent_dst_i, false});
+                        {&tent_dst_i, false}, i);
             }
         }
-
         return status::success;
     }
 
 private:
-    const pd_t *pd() const { return (const pd_t *)primitive_impl_t::pd(); }
-    std::vector<primitive_t *> reorders_;
+    const pd_t *pd() const { return (const pd_t *)primitive_t::pd().get(); }
+    std::vector<std::shared_ptr<primitive_t>> reorders_;
 };
 
 } // namespace cpu
