@@ -19,7 +19,7 @@
 
 #include "dnnl.h"
 
-#include "src/common/dnnl_thread.hpp"
+#include "tests/test_thread.hpp"
 
 #include "dnn_types.hpp"
 #include "dnnl_common.hpp"
@@ -30,7 +30,9 @@
 
 namespace binary {
 
-static int init_pd(const prb_t *p, dnnl_primitive_desc_t &bpd, res_t *r) {
+static int init_pd(dnnl_engine_t engine, const prb_t *p,
+        dnnl_primitive_desc_t &bpd, res_t *r, dir_t dir,
+        const_dnnl_primitive_desc_t hint) {
     dnnl_binary_desc_t bd;
     std::vector<dnnl_memory_desc_t> src_d;
     src_d.resize(p->n_inputs());
@@ -59,15 +61,15 @@ static int init_pd(const prb_t *p, dnnl_primitive_desc_t &bpd, res_t *r) {
                      p->sdims[0].data(), p->ddt, dnnl_format_tag_any),
             WARN);
 
-    dnnl_alg_kind_t alg = alg2alg_kind(p->alg);
+    dnnl_alg_kind_t alg = attr_t::post_ops_t::kind2dnnl_kind(p->alg);
 
     DNN_SAFE(dnnl_binary_desc_init(&bd, alg, &src_d[0], &src_d[1], &dst_d),
             WARN);
 
     auto dnnl_attr = create_dnnl_attr(p->attr);
 
-    dnnl_status_t init_status = dnnl_primitive_desc_create(
-            &bpd, &bd, dnnl_attr, engine_tgt, NULL);
+    dnnl_status_t init_status
+            = dnnl_primitive_desc_create(&bpd, &bd, dnnl_attr, engine, NULL);
 
     dnnl_primitive_attr_destroy(dnnl_attr);
 
@@ -76,8 +78,8 @@ static int init_pd(const prb_t *p, dnnl_primitive_desc_t &bpd, res_t *r) {
     else
         SAFE(init_status, WARN);
 
-    const char *impl_str = query_impl_info(bpd);
-    BENCHDNN_PRINT(5, "oneDNN implementation: %s\n", impl_str);
+    r->impl_name = query_impl_info(bpd);
+    BENCHDNN_PRINT(5, "oneDNN implementation: %s\n", r->impl_name.c_str());
 
     return OK;
 }
@@ -128,10 +130,9 @@ static int compare(const prb_t *p, const dnn_mem_t &fp_mem,
     return r->state == FAILED ? FAIL : OK;
 }
 
-int fill_src(
-        const prb_t *p, int input_idx, dnn_mem_t &mem_dt, dnn_mem_t &mem_fp) {
+int fill_src(int input_idx, dnn_mem_t &mem_dt, dnn_mem_t &mem_fp) {
     const auto nelems = mem_fp.nelems();
-    const auto dt = p->sdt[input_idx];
+    const auto dt = mem_dt.dt();
     const int range = 16;
     const int f_min = dt == dnnl_u8 ? 0 : -range / 2;
 
@@ -140,7 +141,7 @@ int fill_src(
         const float value = (dt == dnnl_bf16 || dt == dnnl_f16)
                 ? (f_min + gen) / range
                 : (f_min + gen) * (1.0f + 4.0f / range);
-        mem_fp.set_elem(i, maybe_saturate(dt, value));
+        mem_fp.set_elem(i, round_to_nearest_representable(dt, value));
     });
 
     SAFE(mem_dt.reorder(mem_fp), WARN);
@@ -148,23 +149,26 @@ int fill_src(
     return OK;
 }
 
+void check_known_skipped_case(const prb_t *p, res_t *r) {
+    check_known_skipped_case_common(p->sdt, r);
+}
+
 int doit(const prb_t *p, res_t *r) {
     if (bench_mode == LIST) return r->state = LISTED, OK;
 
-    dnnl_primitive_desc_t bpd;
-    SAFE(init_pd(p, bpd, r), WARN);
-    if (r->state == SKIPPED || r->state == UNIMPLEMENTED) return OK;
+    check_known_skipped_case(p, r);
+    if (r->state == SKIPPED) return OK;
 
-    dnnl_primitive_t b;
-    DNN_SAFE(dnnl_primitive_create(&b, bpd), WARN);
-    DNN_SAFE(dnnl_primitive_desc_destroy(bpd), CRIT);
+    dnnl_primitive_t b {};
+    SAFE(init_prim(&b, init_pd, p, r), WARN);
+    if (r->state == SKIPPED || r->state == UNIMPLEMENTED) return OK;
 
     const_dnnl_primitive_desc_t const_pd;
     DNN_SAFE(dnnl_primitive_get_primitive_desc(b, &const_pd), CRIT);
 
     if (dnn_mem_t::check_mem_size(const_pd) != OK) {
         DNN_SAFE_V(dnnl_primitive_destroy(b));
-        return r->state = SKIPPED, OK;
+        return r->state = SKIPPED, r->reason = NOT_ENOUGH_RAM, OK;
     }
 
     const auto q = [&](int index = 0) -> const dnnl_memory_desc_t & {
@@ -179,26 +183,28 @@ int doit(const prb_t *p, res_t *r) {
     const auto fp = dnnl_f32;
     const auto tag = get_abx_tag(p->ndims[0]);
 
-    dnn_mem_t src0_fp(src0_md, fp, tag, engine_tgt);
-    dnn_mem_t src0_dt(src0_md, engine_tgt);
-    SAFE(fill_src(p, 0, src0_dt, src0_fp), WARN);
+    const auto &test_engine = get_test_engine();
 
-    dnn_mem_t src1_fp(src1_md, fp, tag, engine_tgt);
-    dnn_mem_t src1_dt(src1_md, engine_tgt);
-    SAFE(fill_src(p, 1, src1_dt, src1_fp), WARN);
+    dnn_mem_t src0_fp(src0_md, fp, tag, test_engine);
+    dnn_mem_t src0_dt(src0_md, test_engine);
+    SAFE(fill_src(0, src0_dt, src0_fp), WARN);
+
+    dnn_mem_t src1_fp(src1_md, fp, tag, test_engine);
+    dnn_mem_t src1_dt(src1_md, test_engine);
+    SAFE(fill_src(1, src1_dt, src1_fp), WARN);
 
     dnn_mem_t &dst_fp = src0_fp; // in-place in ref code
     dnn_mem_t placeholder_dst_dt;
     if (!p->inplace) {
         const auto &dst_md = q(DNNL_ARG_DST);
-        placeholder_dst_dt = dnn_mem_t(dst_md, engine_tgt);
+        placeholder_dst_dt = dnn_mem_t(dst_md, test_engine);
 
         if (p->attr.post_ops.find(attr_t::post_ops_t::kind_t::SUM) >= 0)
             SAFE(placeholder_dst_dt.reorder(dst_fp), WARN);
     }
     dnn_mem_t &dst_dt = p->inplace ? src0_dt : placeholder_dst_dt;
 
-    dnn_mem_t scratchpad_dt(scratchpad_md, engine_tgt);
+    dnn_mem_t scratchpad_dt(scratchpad_md, test_engine);
 
     args_t args;
     args.set(DNNL_ARG_SRC_0, src0_dt);
@@ -206,11 +212,11 @@ int doit(const prb_t *p, res_t *r) {
     args.set(DNNL_ARG_DST, dst_dt);
     args.set(DNNL_ARG_SCRATCHPAD, scratchpad_dt);
 
-    DNN_SAFE(execute_and_wait(b, stream_tgt, args), WARN);
+    SAFE(execute_and_wait(b, args), WARN);
 
     if (bench_mode & CORR) {
         compute_ref(p, src0_fp, src1_fp, dst_fp);
-        dnn_mem_t dst(dst_dt, fp, tag, engine_tgt);
+        dnn_mem_t dst(dst_dt, fp, tag, test_engine);
         SAFE(compare(p, dst_fp, dst, r), WARN);
     }
 
