@@ -21,7 +21,7 @@
 
 #include "dnnl.h"
 
-#include "src/common/dnnl_thread.hpp"
+#include "tests/test_thread.hpp"
 
 #include "dnnl_common.hpp"
 #include "dnnl_memory.hpp"
@@ -58,7 +58,9 @@ inline int transpose_data_wei(
     return OK;
 }
 
-inline int init_pd(const prb_t *p, dnnl_primitive_desc_t &dpd, res_t *r) {
+static int init_pd(dnnl_engine_t engine, const prb_t *p,
+        dnnl_primitive_desc_t &dpd, res_t *r, dir_t dir,
+        const_dnnl_primitive_desc_t hint) {
     dnnl_deconvolution_desc_t cd;
     dnnl_memory_desc_t src_d, wei_d, bia_d, dst_d;
     dnnl_dims_t src_1d_dims = {p->mb, p->ic, p->iw};
@@ -97,15 +99,7 @@ inline int init_pd(const prb_t *p, dnnl_primitive_desc_t &dpd, res_t *r) {
     dnnl_dim_t strides_nd[] = {p->sd, p->sh, p->sw};
     dnnl_dim_t dilates_nd[] = {p->dd, p->dh, p->dw};
     dnnl_dim_t padding_nd[] = {p->pd, p->ph, p->pw};
-
-    auto bph = [&](int64_t ih, int64_t oh, int64_t kh, int64_t sh, int64_t ph,
-                       int64_t dh) {
-        return (oh - 1) * sh - ih + ((kh - 1) * (dh + 1) + 1) - ph;
-    };
-
-    dnnl_dim_t padding_r_nd[] = {bph(p->od, p->id, p->kd, p->sd, p->pd, p->dd),
-            bph(p->oh, p->ih, p->kh, p->sh, p->ph, p->dh),
-            bph(p->ow, p->iw, p->kw, p->sw, p->pw, p->dw)};
+    dnnl_dim_t padding_r_nd[] = {p->pd_r, p->ph_r, p->pw_r};
 
     dnnl_dim_t *strides = strides_nd + (5 - p->ndims);
     dnnl_dim_t *dilates = dilates_nd + (5 - p->ndims);
@@ -150,8 +144,8 @@ inline int init_pd(const prb_t *p, dnnl_primitive_desc_t &dpd, res_t *r) {
 
     auto dnnl_attr = create_dnnl_attr(p->attr, p->oc, p->scales);
 
-    dnnl_status_t init_status = dnnl_primitive_desc_create(
-            &dpd, &cd, dnnl_attr, engine_tgt, NULL);
+    dnnl_status_t init_status
+            = dnnl_primitive_desc_create(&dpd, &cd, dnnl_attr, engine, NULL);
 
     dnnl_primitive_attr_destroy(dnnl_attr);
 
@@ -160,21 +154,47 @@ inline int init_pd(const prb_t *p, dnnl_primitive_desc_t &dpd, res_t *r) {
     }
     SAFE(init_status, WARN);
 
-    const char *impl_str = query_impl_info(dpd);
-    if (maybe_skip(impl_str)) {
-        BENCHDNN_PRINT(2, "SKIPPED: oneDNN implementation: %s\n", impl_str);
+    r->impl_name = query_impl_info(dpd);
+    if (maybe_skip(r->impl_name)) {
+        BENCHDNN_PRINT(2, "SKIPPED: oneDNN implementation: %s\n",
+                r->impl_name.c_str());
         DNN_SAFE(dnnl_primitive_desc_destroy(dpd), WARN);
-        return r->state = SKIPPED, OK;
+        return r->state = SKIPPED, r->reason = SKIP_IMPL_HIT, OK;
     } else {
-        BENCHDNN_PRINT(5, "oneDNN implementation: %s\n", impl_str);
+        BENCHDNN_PRINT(5, "oneDNN implementation: %s\n", r->impl_name.c_str());
     }
 
     return OK;
 }
 
+void check_known_skipped_case(const prb_t *p, res_t *r) {
+    check_known_skipped_case_common(
+            {p->cfg[SRC].dt, p->cfg[WEI].dt, p->cfg[DST].dt}, r);
+    if (r->state == SKIPPED) return;
+
+    // TODO: shapes with dilation and non-unit stride go to reference which
+    // does not support attributes yet. Remove the whole condition once
+    // attributes support is added to reference.
+    // TODO: uncomment deconv shape mb96ic64ih6oc32oh14kh5ph2dh2n"4d/5x5"
+    static auto isa = dnnl_get_effective_cpu_isa();
+    static bool has_avx512_bw = isa >= dnnl_cpu_isa_avx512_core;
+    bool is_int8 = p->cfg[WEI].dt == dnnl_s8;
+    bool has_attr_support = IMPLICATION(is_int8, has_avx512_bw)
+            && IMPLICATION(p->dd != 0, p->sd == 1)
+            && IMPLICATION(p->dh != 0, p->sh == 1)
+            && IMPLICATION(p->dw != 0, p->sw == 1);
+    if (!p->attr.is_def() && !has_attr_support) {
+        r->state = SKIPPED, r->reason = CASE_NOT_SUPPORTED;
+        return;
+    }
+}
+
 int doit(const prb_t *p, res_t *r) {
     if (bench_mode == LIST) return r->state = LISTED, OK;
 
+    check_known_skipped_case(p, r);
+    if (r->state == SKIPPED) return OK;
+
     prb_t p_tr((desc_t)*p, p->dir, p->cfg, p->stag, p->wtag, p->dtag, p->alg,
             p->attr, p->mb, true);
     swap(p_tr.ic, p_tr.oc);
@@ -182,20 +202,16 @@ int doit(const prb_t *p, res_t *r) {
     swap(p_tr.id, p_tr.od);
     swap(p_tr.iw, p_tr.ow);
 
-    dnnl_primitive_desc_t dpd;
-    SAFE(init_pd(p, dpd, r), WARN);
+    dnnl_primitive_t d {};
+    SAFE(init_prim(&d, init_pd, p, r), WARN);
     if (r->state == SKIPPED || r->state == UNIMPLEMENTED) return OK;
 
-    dnnl_primitive_t d;
-    DNN_SAFE(dnnl_primitive_create(&d, dpd), WARN);
-    DNN_SAFE(dnnl_primitive_desc_destroy(dpd), CRIT);
-
     const_dnnl_primitive_desc_t const_pd;
     DNN_SAFE(dnnl_primitive_get_primitive_desc(d, &const_pd), CRIT);
 
     if (dnn_mem_t::check_mem_size(const_pd) != OK) {
         DNN_SAFE_V(dnnl_primitive_destroy(d));
-        return r->state = SKIPPED, OK;
+        return r->state = SKIPPED, r->reason = NOT_ENOUGH_RAM, OK;
     }
 
     const auto q = [&](int index = 0) -> const dnnl_memory_desc_t & {
@@ -221,17 +237,19 @@ int doit(const prb_t *p, res_t *r) {
     const auto src_tag = get_abx_tag(src_md.ndims);
     const auto wei_tag = get_abx_tag(wei_md.ndims);
 
-    dnn_mem_t src_dt(src_md, engine_tgt);
-    dnn_mem_t wei_dt(wei_md, engine_tgt);
-    dnn_mem_t dst_dt(dst_md, engine_tgt);
-    dnn_mem_t bia_dt(bia_md, engine_tgt);
-    dnn_mem_t scratchpad_dt(scratchpad_md, engine_tgt);
+    const auto &test_engine = get_test_engine();
 
-    dnn_mem_t src_fp(src_md, fp, src_tag, engine_tgt);
-    dnn_mem_t wei_fp(wei_md, fp, wei_tag, engine_tgt);
-    dnn_mem_t dst_fp(dst_md, fp, src_tag, engine_tgt);
-    dnn_mem_t wei_tr_fp(wei_tr_md, fp, wei_tag, engine_tgt);
-    dnn_mem_t bia_fp(bia_md, fp, dnnl_x, engine_tgt);
+    dnn_mem_t src_dt(src_md, test_engine);
+    dnn_mem_t wei_dt(wei_md, test_engine);
+    dnn_mem_t dst_dt(dst_md, test_engine);
+    dnn_mem_t bia_dt(bia_md, test_engine);
+    dnn_mem_t scratchpad_dt(scratchpad_md, test_engine);
+
+    dnn_mem_t src_fp(src_md, fp, src_tag, test_engine);
+    dnn_mem_t wei_fp(wei_md, fp, wei_tag, test_engine);
+    dnn_mem_t dst_fp(dst_md, fp, src_tag, test_engine);
+    dnn_mem_t wei_tr_fp(wei_tr_md, fp, wei_tag, test_engine);
+    dnn_mem_t bia_fp(bia_md, fp, dnnl_x, test_engine);
 
     /* fill memory + reorders <-> */
     SAFE(fill_dst(p, dst_dt, dst_fp, r), WARN);
@@ -249,12 +267,12 @@ int doit(const prb_t *p, res_t *r) {
         args.set(DNNL_ARG_DST, dst_dt);
         args.set(DNNL_ARG_SCRATCHPAD, scratchpad_dt);
 
-        DNN_SAFE(execute_and_wait(d, stream_tgt, args), WARN);
+        SAFE(execute_and_wait(d, args), WARN);
 
         if (bench_mode & CORR) {
             compute_ref_bwd_d(
                     &p_tr, nullptr, dst_fp, wei_tr_fp, bia_fp, src_fp);
-            dnn_mem_t dst(dst_dt, fp, src_tag, engine_tgt);
+            dnn_mem_t dst(dst_dt, fp, src_tag, test_engine);
             SAFE(compare_dst(p, dst, dst_fp, r, true), WARN);
         }
     } else if (p->dir == BWD_D) {
@@ -263,12 +281,12 @@ int doit(const prb_t *p, res_t *r) {
         args.set(DNNL_ARG_DIFF_SRC, src_dt);
         args.set(DNNL_ARG_SCRATCHPAD, scratchpad_dt);
 
-        DNN_SAFE(execute_and_wait(d, stream_tgt, args), WARN);
+        SAFE(execute_and_wait(d, args), WARN);
 
         if (bench_mode & CORR) {
             dnn_mem_t zero_fp;
             compute_ref_fwd(&p_tr, nullptr, dst_fp, wei_tr_fp, zero_fp, src_fp);
-            dnn_mem_t src(src_dt, fp, src_tag, engine_tgt);
+            dnn_mem_t src(src_dt, fp, src_tag, test_engine);
             SAFE(compare_src(p, src, src_fp, r, true), WARN);
         }
     } else if (p->dir & FLAG_BWD && p->dir & FLAG_WEI) {
@@ -278,16 +296,16 @@ int doit(const prb_t *p, res_t *r) {
         args.set(DNNL_ARG_DIFF_BIAS, bia_dt);
         args.set(DNNL_ARG_SCRATCHPAD, scratchpad_dt);
 
-        DNN_SAFE(execute_and_wait(d, stream_tgt, args), WARN);
+        SAFE(execute_and_wait(d, args), WARN);
 
         if (bench_mode & CORR) {
             compute_ref_bwd_weights(&p_tr, dst_fp, wei_tr_fp, src_fp);
             transpose_data_wei(&p_tr, wei_tr_fp, wei_fp);
-            dnn_mem_t wei(wei_dt, fp, wei_tag, engine_tgt);
+            dnn_mem_t wei(wei_dt, fp, wei_tag, test_engine);
             SAFE(compare_wei(&p_tr, wei, wei_fp, r, true), WARN);
             if (p->dir & FLAG_BIA) {
                 compute_ref_bwd_bias(p, bia_fp, dst_fp);
-                dnn_mem_t bia(bia_dt, fp, dnnl_x, engine_tgt);
+                dnn_mem_t bia(bia_dt, fp, dnnl_x, test_engine);
                 SAFE(compare_bia(p, bia, bia_fp, r, true), WARN);
             }
         }
