@@ -21,7 +21,7 @@
 
 #include "dnnl.h"
 
-#include "src/common/dnnl_thread.hpp"
+#include "tests/test_thread.hpp"
 
 #include "dnnl_common.hpp"
 #include "dnnl_memory.hpp"
@@ -135,8 +135,9 @@ int fill_ws(const prb_t *p, dnn_mem_t &mem_dt, dnn_mem_t &mem_fp, res_t *r) {
     return OK;
 }
 
-int init_pd(const prb_t *p, dir_t dir, dnnl_primitive_desc_t &ppd,
-        const_dnnl_primitive_desc_t hint, res_t *r) {
+static int init_pd(dnnl_engine_t engine, const prb_t *p,
+        dnnl_primitive_desc_t &ppd, res_t *r, dir_t dir,
+        const_dnnl_primitive_desc_t hint) {
     dnnl_memory_desc_t src_d, dst_d;
 
     dnnl_dims_t src_1d_dims = {p->mb, p->ic, p->iw};
@@ -198,8 +199,8 @@ int init_pd(const prb_t *p, dir_t dir, dnnl_primitive_desc_t &ppd,
 
     auto dnnl_attr = create_dnnl_attr(attr_t());
 
-    dnnl_status_t init_status = dnnl_primitive_desc_create(
-            &ppd, &pd, dnnl_attr, engine_tgt, hint);
+    dnnl_status_t init_status
+            = dnnl_primitive_desc_create(&ppd, &pd, dnnl_attr, engine, hint);
 
     dnnl_primitive_attr_destroy(dnnl_attr);
 
@@ -211,44 +212,39 @@ int init_pd(const prb_t *p, dir_t dir, dnnl_primitive_desc_t &ppd,
     // Return if pd is not the one being tested
     if ((dir & FLAG_FWD) != (p->dir & FLAG_FWD)) return OK;
 
-    const char *impl_str = query_impl_info(ppd);
-    if (maybe_skip(impl_str)) {
-        BENCHDNN_PRINT(2, "SKIPPED: oneDNN implementation: %s\n", impl_str);
+    r->impl_name = query_impl_info(ppd);
+    if (maybe_skip(r->impl_name)) {
+        BENCHDNN_PRINT(2, "SKIPPED: oneDNN implementation: %s\n",
+                r->impl_name.c_str());
         DNN_SAFE(dnnl_primitive_desc_destroy(ppd), WARN);
-        return r->state = SKIPPED, OK;
+        return r->state = SKIPPED, r->reason = SKIP_IMPL_HIT, OK;
     } else {
-        BENCHDNN_PRINT(5, "oneDNN implementation: %s\n", impl_str);
+        BENCHDNN_PRINT(5, "oneDNN implementation: %s\n", r->impl_name.c_str());
     }
 
     return OK;
 }
 
-int init_pd_fwd(const prb_t *p, dnnl_primitive_desc_t &ppd, res_t *r) {
-    return init_pd(p, FLAG_FWD, ppd, nullptr, r);
-}
-
-int init_pd_bwd(const prb_t *p, dnnl_primitive_desc_t &ppd,
-        const_dnnl_primitive_desc_t hint, res_t *r) {
-    return init_pd(p, FLAG_BWD, ppd, hint, r);
+void check_known_skipped_case(const prb_t *p, res_t *r) {
+    check_known_skipped_case_common({p->cfg[SRC].dt, p->cfg[DST].dt}, r);
 }
 
 int doit(const prb_t *p, res_t *r) {
     if (bench_mode == LIST) return r->state = LISTED, OK;
 
-    dnnl_primitive_desc_t ppd;
-    SAFE(init_pd_fwd(p, ppd, r), WARN);
-    if (r->state == SKIPPED || r->state == UNIMPLEMENTED) return OK;
+    check_known_skipped_case(p, r);
+    if (r->state == SKIPPED) return OK;
 
-    dnnl_primitive_t pp;
-    DNN_SAFE(dnnl_primitive_create(&pp, ppd), WARN);
-    DNN_SAFE(dnnl_primitive_desc_destroy(ppd), CRIT);
+    dnnl_primitive_t pp {};
+    SAFE(init_prim(&pp, init_pd, p, r), WARN);
+    if (r->state == SKIPPED || r->state == UNIMPLEMENTED) return OK;
 
     const_dnnl_primitive_desc_t const_fpd;
     DNN_SAFE(dnnl_primitive_get_primitive_desc(pp, &const_fpd), CRIT);
 
     if (dnn_mem_t::check_mem_size(const_fpd) != OK) {
         DNN_SAFE_V(dnnl_primitive_destroy(pp));
-        return r->state = SKIPPED, OK;
+        return r->state = SKIPPED, r->reason = NOT_ENOUGH_RAM, OK;
     }
 
     const auto q = [](const_dnnl_primitive_desc_t pd,
@@ -266,80 +262,81 @@ int doit(const prb_t *p, res_t *r) {
     const auto fp = dnnl_f32;
     const auto tag = get_abx_tag(p->ndims);
 
-    dnn_mem_t src_fp(src_md, fp, tag, engine_tgt);
-    dnn_mem_t src_dt(src_md, engine_tgt);
+    const auto &test_engine = get_test_engine();
 
-    dnn_mem_t dst_fp(dst_md, fp, tag, engine_tgt);
-    dnn_mem_t dst_dt(dst_md, engine_tgt);
+    dnn_mem_t src_fp(src_md, fp, tag, test_engine);
+    dnn_mem_t src_dt(src_md, test_engine);
+
+    dnn_mem_t dst_fp(dst_md, fp, tag, test_engine);
+    dnn_mem_t dst_dt(dst_md, test_engine);
 
     if (p->dir & FLAG_INF) SAFE(ws_md.ndims == 0 ? OK : FAIL, WARN);
-    dnn_mem_t ws_fp(ws_md, engine_tgt);
-    dnn_mem_t ws_dt(ws_md, engine_tgt);
-    dnn_mem_t scratchpad_dt(scratchpad_md, engine_tgt);
+    dnn_mem_t ws_fp(ws_md, test_engine);
+    dnn_mem_t ws_dt(ws_md, test_engine);
+    dnn_mem_t scratchpad_dt(scratchpad_md, test_engine);
 
     dnn_mem_t d_src_dt, d_dst_dt;
 
     SAFE(fill_src(p, src_dt, src_fp, r), WARN);
 
-    args_t args_fwd, args_bwd;
-    args_fwd.set(DNNL_ARG_SRC, src_dt);
-    args_fwd.set(DNNL_ARG_DST, dst_dt);
-    args_fwd.set(DNNL_ARG_WORKSPACE, ws_dt);
-    args_fwd.set(DNNL_ARG_SCRATCHPAD, scratchpad_dt);
-    args_t &args = args_fwd;
+    args_t args;
+    args.set(DNNL_ARG_SRC, src_dt);
+    args.set(DNNL_ARG_DST, dst_dt);
+    args.set(DNNL_ARG_WORKSPACE, ws_dt);
+    args.set(DNNL_ARG_SCRATCHPAD, scratchpad_dt);
 
-    DNN_SAFE(execute_and_wait(pp, stream_tgt, args), WARN);
+    SAFE(execute_and_wait(pp, args), WARN);
 
     // want this pass on backward to get ws_fp filled properly
     if (bench_mode & CORR) {
         compute_ref_fwd(p, src_fp, dst_fp, ws_fp);
         if (p->dir & FLAG_FWD) {
-            dnn_mem_t dst(dst_dt, fp, tag, engine_tgt);
+            dnn_mem_t dst(dst_dt, fp, tag, test_engine);
             SAFE(compare_dst(p, dst, dst_fp, r), WARN);
         }
     }
 
     if (p->dir & FLAG_BWD) {
-        SAFE(init_pd_bwd(p, ppd, const_fpd, r), WARN);
-        DNN_SAFE(dnnl_primitive_destroy(pp), CRIT);
+        dnnl_primitive_t bwd_p {};
+        int status = init_prim(&bwd_p, init_pd, p, r, FLAG_BWD, const_fpd);
+        dnnl_primitive_destroy(pp);
+        if (status != OK) return status;
         if (r->state == SKIPPED || r->state == UNIMPLEMENTED) return OK;
-
-        DNN_SAFE(dnnl_primitive_create(&pp, ppd), WARN);
-        DNN_SAFE(dnnl_primitive_desc_destroy(ppd), CRIT);
+        pp = bwd_p;
 
         const_dnnl_primitive_desc_t const_bpd;
         DNN_SAFE(dnnl_primitive_get_primitive_desc(pp, &const_bpd), CRIT);
 
         if (dnn_mem_t::check_mem_size(const_bpd) != OK) {
             DNN_SAFE_V(dnnl_primitive_destroy(pp));
-            return r->state = SKIPPED, OK;
+            return r->state = SKIPPED, r->reason = NOT_ENOUGH_RAM, OK;
         }
 
         const auto &d_dst_md = q(const_bpd, DNNL_ARG_DIFF_DST);
         const auto &d_src_md = q(const_bpd, DNNL_ARG_DIFF_SRC);
         const auto &d_scratchpad_md = q(const_bpd, DNNL_ARG_SCRATCHPAD);
 
-        dnn_mem_t d_dst_fp = dnn_mem_t(d_dst_md, fp, tag, engine_tgt);
-        d_dst_dt = dnn_mem_t(d_dst_md, p->cfg[DST].dt, engine_tgt);
+        dnn_mem_t d_dst_fp = dnn_mem_t(d_dst_md, fp, tag, test_engine);
+        d_dst_dt = dnn_mem_t(d_dst_md, p->cfg[DST].dt, test_engine);
 
-        dnn_mem_t d_src_fp = dnn_mem_t(d_src_md, fp, tag, engine_tgt);
-        d_src_dt = dnn_mem_t(d_src_md, p->cfg[SRC].dt, engine_tgt);
+        dnn_mem_t d_src_fp = dnn_mem_t(d_src_md, fp, tag, test_engine);
+        d_src_dt = dnn_mem_t(d_src_md, p->cfg[SRC].dt, test_engine);
 
-        scratchpad_dt = dnn_mem_t(d_scratchpad_md, engine_tgt);
+        scratchpad_dt = dnn_mem_t(d_scratchpad_md, test_engine);
 
         SAFE(fill_dst(p, d_dst_dt, d_dst_fp, r), WARN);
 
-        args_bwd.set(DNNL_ARG_DIFF_DST, d_dst_dt);
-        args_bwd.set(DNNL_ARG_DIFF_SRC, d_src_dt);
-        args_bwd.set(DNNL_ARG_WORKSPACE, ws_dt);
-        args_bwd.set(DNNL_ARG_SCRATCHPAD, scratchpad_dt);
-        args = args_bwd;
+        args.clear();
+        args.set(DNNL_ARG_DIFF_DST, d_dst_dt);
+        args.set(DNNL_ARG_DIFF_SRC, d_src_dt);
+        args.set(DNNL_ARG_WORKSPACE, ws_dt);
+        args.set(DNNL_ARG_SCRATCHPAD, scratchpad_dt);
 
-        DNN_SAFE(execute_and_wait(pp, stream_tgt, args), WARN);
+        SAFE(execute_and_wait(pp, args), WARN);
 
         if (bench_mode & CORR) {
             compute_ref_bwd(p, d_src_fp, d_dst_fp, ws_fp);
-            dnn_mem_t diff_src(d_src_dt, fp, tag, engine_tgt);
+            dnn_mem_t diff_src(d_src_dt, fp, tag, test_engine);
             SAFE(compare_src(p, diff_src, d_src_fp, r), WARN);
         }
     }
