@@ -29,19 +29,7 @@ namespace reorder {
 
 // prepare the output scales and mask
 int prepare_attr_bundle(const prb_t *p, attr_bundle_t &attr_bundle) {
-    auto get_scale_mask = [](const attr_t &attr) {
-        using P = attr_t::scale_t::policy_t;
-        switch (attr.oscale.policy) {
-            case P::PER_DIM_0: return (1 << 0);
-            case P::PER_DIM_1: return (1 << 1);
-            case P::PER_DIM_01: return (1 << 0) + (1 << 1);
-            case P::COMMON:
-            case P::NONE: return 0;
-            default: SAFE_V(FAIL); return 0;
-        }
-    };
-
-    const int mask = get_scale_mask(p->attr);
+    const int mask = attr_t::scale_t::get_default_mask(p->attr.oscale.policy);
 
     int64_t uniq_scales = 1;
     for (int d = 0; d < p->ndims; ++d)
@@ -223,16 +211,21 @@ static int compare(const prb_t *p, const dnn_mem_t &mem_ref,
                            attr_t::post_ops_t::kind_t::SUM)
                         == -1;
 
-        bool mistrusted = reg == 0 || (check_inf_p && inf_p == 0)
+        bool mistrusted = (check_inf_p && inf_p == 0)
                 || (check_inf_n && inf_n == 0) || (check_zeros && zeros == 0);
+
+        bool expect_regular = max_scale < 2e9 || dt_out == dnnl_f32;
+        if (expect_regular) mistrusted = mistrusted || reg == 0;
+
         if (mistrusted) r->state = MISTRUSTED;
     }
 
     return r->state == FAILED ? FAIL : OK;
 }
 
-static int init_pd(const prb_t *p, dnnl_primitive_desc_t &rpd,
-        const attr_bundle_t &attr_bundle, res_t *r) {
+static int init_pd_custom(dnnl_engine_t engine, const prb_t *p,
+        dnnl_primitive_desc_t &rpd, const attr_bundle_t &attr_bundle,
+        res_t *r) {
     const auto &rc = p->reorder;
     auto dims = rc.dims;
     for (int d = 0; d < p->ndims; ++d)
@@ -251,19 +244,27 @@ static int init_pd(const prb_t *p, dnnl_primitive_desc_t &rpd,
     fill_memory_extra(p, dst_md_extra);
     dst_d.extra = dst_md_extra;
 
-    dnnl_status_t init_status = dnnl_reorder_primitive_desc_create(&rpd, &src_d,
-            engine_tgt, &dst_d, engine_tgt, attr_bundle.dnnl_attr());
+    dnnl_status_t init_status = dnnl_reorder_primitive_desc_create(
+            &rpd, &src_d, engine, &dst_d, engine, attr_bundle.dnnl_attr());
     if (init_status == dnnl_unimplemented) return r->state = UNIMPLEMENTED, OK;
     SAFE(init_status, WARN);
 
-    const char *impl_str = query_impl_info(rpd);
-    BENCHDNN_PRINT(5, "oneDNN implementation: %s\n", impl_str);
+    r->impl_name = query_impl_info(rpd);
+    BENCHDNN_PRINT(5, "oneDNN implementation: %s\n", r->impl_name.c_str());
 
     return OK;
 }
 
+void check_known_skipped_case(const prb_t *p, res_t *r) {
+    check_known_skipped_case_common({p->conf_in->dt, p->conf_out->dt}, r);
+}
+
 int doit(const prb_t *p, res_t *r) {
     if (bench_mode == LIST) return r->state = LISTED, OK;
+
+    check_known_skipped_case(p, r);
+    if (r->state == SKIPPED) return OK;
+
     //                                       ___________________
     //                                      |                   |
     //                                      | performance timer |
@@ -294,20 +295,25 @@ int doit(const prb_t *p, res_t *r) {
     SAFE(prepare_attr_bundle(p, attr_bundle), WARN);
 
     /* Step 2: create target reorder primitive */
-    dnnl_primitive_desc_t rpd;
-    SAFE(init_pd(p, rpd, attr_bundle, r), WARN);
-    if (r->state == SKIPPED || r->state == UNIMPLEMENTED) return OK;
+    dnnl_primitive_t rp {};
+    // TODO: align init_pd interface with a common one which is used
+    // in the rest of the benchdnn drivers
+    auto init_pd = [&](dnnl_engine_t engine, const prb_t *p,
+                           dnnl_primitive_desc_t &rpd, res_t *r, dir_t dir,
+                           const_dnnl_primitive_desc_t hint) {
+        SAFE(init_pd_custom(engine, p, rpd, attr_bundle, r), WARN);
+        return OK;
+    };
 
-    dnnl_primitive_t rp;
-    DNN_SAFE(dnnl_primitive_create(&rp, rpd), WARN);
-    DNN_SAFE(dnnl_primitive_desc_destroy(rpd), CRIT);
+    SAFE(init_prim(&rp, init_pd, p, r), WARN);
+    if (r->state == SKIPPED || r->state == UNIMPLEMENTED) return OK;
 
     const_dnnl_primitive_desc_t const_pd;
     DNN_SAFE(dnnl_primitive_get_primitive_desc(rp, &const_pd), CRIT);
 
     if (dnn_mem_t::check_mem_size(const_pd) != OK) {
         DNN_SAFE_V(dnnl_primitive_destroy(rp));
-        return r->state = SKIPPED, OK;
+        return r->state = SKIPPED, r->reason = NOT_ENOUGH_RAM, OK;
     }
 
     const auto q = [&](int index = 0) -> const dnnl_memory_desc_t & {
@@ -336,12 +342,14 @@ int doit(const prb_t *p, res_t *r) {
     const auto src_dt = src_md.data_type;
     const auto dst_dt = dst_md.data_type;
 
-    dnn_mem_t src_dt_in_fmt_ref(src_md, src_dt, tag, engine_tgt);
-    dnn_mem_t src_dt_in_fmt_in(src_md, engine_tgt);
+    const auto &test_engine = get_test_engine();
 
-    dnn_mem_t dst_dt_out_fmt_ref(dst_md, dst_dt, tag, engine_tgt);
-    dnn_mem_t dst_dt_out_fmt_out(dst_md, engine_tgt);
-    dnn_mem_t scratchpad_dt(scratchpad_md, engine_tgt);
+    dnn_mem_t src_dt_in_fmt_ref(src_md, src_dt, tag, test_engine);
+    dnn_mem_t src_dt_in_fmt_in(src_md, test_engine);
+
+    dnn_mem_t dst_dt_out_fmt_ref(dst_md, dst_dt, tag, test_engine);
+    dnn_mem_t dst_dt_out_fmt_out(dst_md, test_engine);
+    dnn_mem_t scratchpad_dt(scratchpad_md, test_engine);
 
     /* Step 4: fill input memory */
     SAFE(fill_memory(p, SRC, src_dt_in_fmt_ref, attr_bundle), WARN);
@@ -355,11 +363,11 @@ int doit(const prb_t *p, res_t *r) {
     }
 
     dnn_mem_t scales, src_zero_points_m, dst_zero_points_m;
-    maybe_prepare_runtime_scales(scales, attr_bundle, engine_tgt);
+    maybe_prepare_runtime_scales(scales, attr_bundle);
     maybe_prepare_runtime_zero_points(
-            src_zero_points_m, attr_bundle.attr, DNNL_ARG_SRC, engine_tgt);
+            src_zero_points_m, attr_bundle.attr, DNNL_ARG_SRC);
     maybe_prepare_runtime_zero_points(
-            dst_zero_points_m, attr_bundle.attr, DNNL_ARG_DST, engine_tgt);
+            dst_zero_points_m, attr_bundle.attr, DNNL_ARG_DST);
 
     args_t args;
     args.set(DNNL_ARG_FROM, src_dt_in_fmt_in);
@@ -369,7 +377,7 @@ int doit(const prb_t *p, res_t *r) {
     args.set(DNNL_ARG_ATTR_ZERO_POINTS | DNNL_ARG_SRC, src_zero_points_m);
     args.set(DNNL_ARG_ATTR_ZERO_POINTS | DNNL_ARG_DST, dst_zero_points_m);
 
-    DNN_SAFE(execute_and_wait(rp, stream_tgt, args), WARN);
+    SAFE(execute_and_wait(rp, args), WARN);
 
     /* Step 6: check correctness */
     if (bench_mode & CORR) {
@@ -381,7 +389,7 @@ int doit(const prb_t *p, res_t *r) {
             /* Step 5a: oneDNN reorder from ref format to output format */
             dnnl_memory_extra_desc_t dst_extra {};
             fill_memory_extra(p, dst_extra);
-            dnn_mem_t ref_dst_dt_out_fmt_out(dst_md, engine_tgt);
+            dnn_mem_t ref_dst_dt_out_fmt_out(dst_md, test_engine);
             ref_dst_dt_out_fmt_out.md_.extra = dst_extra;
 
             SAFE(ref_dst_dt_out_fmt_out.reorder(src_dt_in_fmt_ref, attr_bundle),
@@ -400,7 +408,7 @@ int doit(const prb_t *p, res_t *r) {
                     WARN);
 
             /* Step 5c: compare benchdnn and oneDNN output */
-            dnn_mem_t dst_dt_out(dst_md, dst_dt, tag, engine_tgt);
+            dnn_mem_t dst_dt_out(dst_md, dst_dt, tag, test_engine);
             SAFE(dst_dt_out.reorder(dst_dt_out_fmt_out), WARN);
             SAFE(compare(p, dst_dt_out_fmt_ref, dst_dt_out, attr_bundle, r),
                     WARN);
