@@ -22,7 +22,7 @@
 
 #include "dnnl.h"
 
-#include "src/common/dnnl_thread.hpp"
+#include "tests/test_thread.hpp"
 
 #include "dnnl_common.hpp"
 #include "dnnl_memory.hpp"
@@ -33,7 +33,8 @@
 
 #define COMPARE_DAT(rc, kind, a, lay) \
     do { \
-        dnn_mem_t CONCAT2(a, _dt_plain)(CONCAT2(a, _dt), fp, lay, engine_tgt); \
+        dnn_mem_t CONCAT2(a, _dt_plain)( \
+                CONCAT2(a, _dt), fp, lay, test_engine); \
         rc |= compare_dat( \
                 p, kind, CONCAT2(a, _dt_plain), CONCAT2(a, _fp), r, true); \
     } while (0)
@@ -51,18 +52,12 @@ void create_dnnl_rnn_attr(const prb_t &p, dnnl_primitive_attr_t *dnnl_attr) {
         DNN_SAFE_V(dnnl_primitive_attr_set_rnn_tparams(*dnnl_attr, true,
                 p.n_gates(), p.linear_scales, p.linear_cscale));
 
-    if (p.scale_policy == policy_t::PER_OC) {
-        DNN_SAFE_V(dnnl_primitive_attr_set_rnn_weights_qparams(
-                *dnnl_attr, p.dhc * p.n_gates(), 0x18, p.wei_oc_scales));
-    } else if (p.scale_policy == policy_t::COMMON && p.wei_scale != 1.) {
-        DNN_SAFE_V(dnnl_primitive_attr_set_rnn_weights_qparams(
-                *dnnl_attr, 1, 0, &p.wei_scale));
-    }
+    DNN_SAFE_V(dnnl_primitive_attr_set_rnn_weights_qparams(
+            *dnnl_attr, p.wei_nscales, p.wei_scales_mask, p.wei_scales));
 
-    if (p.data_scale != 1.0 || p.data_shift != 0.0) {
+    if (p.data_scale != 1.0 || p.data_shift != 0.0)
         DNN_SAFE_V(dnnl_primitive_attr_set_rnn_data_qparams(
                 *dnnl_attr, p.data_scale, p.data_shift));
-    }
 
     DNN_SAFE_V(dnnl_primitive_attr_set_scratchpad_mode(
             *dnnl_attr, scratchpad_mode));
@@ -89,27 +84,26 @@ int check_s8s8_reorder(const prb_t &p, dnn_mem_t &mem_dt, dnn_mem_t &mem_fp) {
     // alignment as packed buffer is aligned internally and the offset
     // is kept in the metadata.
     // Works fine with dnn_mem_t as it is align to 2MB large page boundary
-    dnn_mem_t mem_s8_src(mem_fp.md_, dnnl_s8, engine_tgt);
-    dnn_mem_t mem_s8_dst(mem_dt.md_, dnnl_s8, engine_tgt);
+    dnn_mem_t mem_s8_src(mem_fp.md_, dnnl_s8, get_test_engine());
+    dnn_mem_t mem_s8_dst(mem_dt.md_, dnnl_s8, get_test_engine());
 
     /* 1. compute f32_plain --quant--> s8_plain_quantized */
     /* Do fixed partitioning to have same filling for any number of threads */
     auto nelems = mem_fp.nelems();
     const int64_t n_chunks = 16;
     const int64_t chunk_size = div_up(nelems, n_chunks);
-    dnnl::impl::parallel_nd(n_chunks, [&](int idx_chunk) {
+    auto quantize = [&](const float *scales, int nscales, int idx_chunk) {
         int64_t idx_start = idx_chunk * chunk_size;
         int64_t idx_end = MIN2(idx_start + chunk_size, nelems);
         for (int64_t idx = idx_start; idx < idx_end; ++idx) {
-            const float current_scale = p.scale_policy == policy_t::PER_OC
-                    ? p.wei_oc_scales[idx % (p.dhc * p.n_gates())]
-                    : p.wei_scale;
+            const float current_scale = scales[idx % nscales];
             float val_f32 = mem_fp.get_elem(idx);
-            //int8_t val_s8 = saturate<dnnl_s8>(val_f32);
             int8_t val_s8 = saturate<dnnl_s8>(val_f32 * current_scale);
             mem_s8_src.set_elem(idx, val_s8);
         }
-    });
+    };
+    dnnl::impl::parallel_nd(n_chunks,
+            [&](int idx) { quantize(p.wei_scales, p.wei_nscales, idx); });
 
     /* 2. compute s8_plain_quantized --reorder--> s8_packed_quantized */
     mem_s8_dst.reorder(mem_s8_src);
@@ -149,27 +143,20 @@ int fill_memory(const prb_t &p, data_kind_t kind, dnn_mem_t &mem_dt,
     //    4.a. If the tensor is weights -- q10n it here;
     //    4.b. If the tensor is data -- reference benchdnn RNN will quantize it.
 
-    // default, nothing to be done
-    float scale = 1.f, shift = 0.f;
-    bool need_recompute_scale = false;
+    // pass rnn attributes to f32 -> int8 reorders only
     const_dnnl_primitive_attr_t reorder_attr = nullptr;
-
-    if (p.is_int8()) {
-        if (kind == WEIGHTS_LAYER || kind == WEIGHTS_ITER) {
-            need_recompute_scale = p.scale_policy == policy_t::PER_OC;
-            if (!need_recompute_scale) scale = p.wei_scale;
-        } else if (dt == dnnl_u8 && (kind == SRC_LAYER || kind == SRC_ITER)) {
-            scale = p.data_scale;
-            shift = p.data_shift;
-        }
-        // pass rnn attributes to f32 -> int8 reorders
-        if (dt != dnnl_f32) reorder_attr = attr;
-    }
+    if (p.is_int8() && (dt != dnnl_f32)) reorder_attr = attr;
+    float default_scales[1] = {1.0f};
+    float default_shift = 0.0f;
 
     /* Do fixed partitioning to have same filling for any number of threads */
     const int64_t n_chunks = 16;
     const int64_t chunk_size = div_up(nelems, n_chunks);
-    dnnl::impl::parallel_nd(n_chunks, [&](int idx_chunk) {
+
+    // 2. We fill first f32 de-quantized data, by inverse-applying the scale
+    //    and shift to the data generated by cfg distribution;
+    auto fill_chunk = [&](const float *scales, int nscales, float shift,
+                              int idx_chunk) {
         int64_t idx_start = idx_chunk * chunk_size;
         int64_t idx_end = MIN2(idx_start + chunk_size, nelems);
         std::minstd_rand msr;
@@ -178,11 +165,8 @@ int fill_memory(const prb_t &p, data_kind_t kind, dnn_mem_t &mem_dt,
         for (int64_t idx = idx_start; idx < idx_end; ++idx) {
             float val = round_to_nearest_representable(dt, gen(msr));
             val = MAX2(MIN2(val, max), min);
-
-            const float current_scale = need_recompute_scale
-                    ? p.wei_oc_scales[idx % (p.dhc * p.n_gates())]
-                    : scale;
-            val = (val - shift) / current_scale; // change only int8-case
+            val = (val - shift)
+                    / scales[idx % nscales]; // change only int8-case
 
             // Vanilla RNN with RELU testing related only: flip the sign of
             // inputs for `mb` == 0 to test RELU part
@@ -193,28 +177,54 @@ int fill_memory(const prb_t &p, data_kind_t kind, dnn_mem_t &mem_dt,
             }
             mem_fp.set_elem(idx, val);
         }
-    });
+    };
+    switch (kind) {
+        case WEIGHTS_LAYER:
+        case WEIGHTS_ITER:
+            dnnl::impl::parallel_nd(n_chunks, [&](int idx) {
+                fill_chunk(p.wei_scales, p.wei_nscales, 0.0f, idx);
+            });
+            break;
+        case SRC_LAYER:
+        case SRC_ITER:
+            dnnl::impl::parallel_nd(n_chunks, [&](int idx) {
+                fill_chunk(&(p.data_scale), 1, p.data_shift, idx);
+            });
+            break;
+        default: // we do no scale/shift
+            dnnl::impl::parallel_nd(n_chunks, [&](int idx) {
+                fill_chunk(default_scales, 1, default_shift, idx);
+            });
+    }
 
+    // 3. We reorder the data for the DNNL RNN primitive
     mem_dt.reorder(mem_fp, {reorder_attr});
     if ((reorder_attr != nullptr) && (dt == dnnl_s8))
         if (check_s8s8_reorder(p, mem_dt, mem_fp) != OK) return FAIL;
 
     // Bullet 4.a holds: quantize weights for int8 benchdnn reference RNN
-    if (p.is_int8() && (kind == WEIGHTS_LAYER || kind == WEIGHTS_ITER)) {
-        dnnl::impl::parallel_nd(n_chunks, [&](int idx_chunk) {
-            int64_t idx_start = idx_chunk * chunk_size;
-            int64_t idx_end = MIN2(idx_start + chunk_size, nelems);
-            for (int64_t idx = idx_start; idx < idx_end; ++idx) {
-                const float current_scale = need_recompute_scale
-                        ? p.wei_oc_scales[idx % (p.dhc * p.n_gates())]
-                        : scale;
-
-                float val = ((float *)mem_fp)[idx];
-                val = round(current_scale * val);
-
-                mem_fp.set_elem(idx, MAX2(MIN2(val, max), min));
-            }
-        });
+    if (p.is_int8()) {
+        auto quantize_chunk
+                = [&](const float *scales, int nscales, int idx_chunk) {
+                      int64_t idx_start = idx_chunk * chunk_size;
+                      int64_t idx_end = MIN2(idx_start + chunk_size, nelems);
+                      for (int64_t idx = idx_start; idx < idx_end; ++idx) {
+                          float current_scale = scales[idx % nscales];
+                          float val = ((float *)mem_fp)[idx];
+                          val = round(current_scale * val);
+                          mem_fp.set_elem(idx, MAX2(MIN2(val, max), min));
+                      }
+                  };
+        switch (kind) {
+            case WEIGHTS_LAYER:
+            case WEIGHTS_ITER:
+                dnnl::impl::parallel_nd(n_chunks, [&](int idx) {
+                    quantize_chunk(p.wei_scales, p.wei_nscales, idx);
+                });
+                break;
+            default: // Nothing to do
+                break;
+        }
     }
 
     return OK;
@@ -375,8 +385,10 @@ int fill_bias(const prb_t &p, data_kind_t kind, dnn_mem_t &mem_dt,
     return OK;
 }
 
-inline int init_pd(
-        const prb_t &p, dnnl_primitive_desc_t &rpd, res_t *r, bool is_fwd) {
+static int init_pd(dnnl_engine_t engine, const prb_t *p_ptr,
+        dnnl_primitive_desc_t &rpd, res_t *r, dir_t dir,
+        const_dnnl_primitive_desc_t hint) {
+    const auto &p = *p_ptr;
     dnnl_rnn_desc_t rd;
     dnnl_prop_kind_t fwd_prop = dnnl_prop_kind_undef;
     switch (p.prop) {
@@ -494,7 +506,7 @@ inline int init_pd(
     // Initializing the forward pass
     // When inference, we use forward_inference
     // When training, we use forward_training
-    if (is_fwd) {
+    if (dir & FLAG_FWD) {
         DNN_SAFE(init_rnn_fwd_desc(&rd, p, fwd_prop, &src_layer_d, &src_iter_d,
                          &src_iter_c_d, &weights_layer_d, &weights_iter_d,
                          &weights_peephole_d, &weights_projection_d, &bias_d,
@@ -564,47 +576,51 @@ inline int init_pd(
 
     dnnl_primitive_attr_t dnnl_attr;
     create_dnnl_rnn_attr(p, &dnnl_attr);
-    dnnl_status_t init_status = dnnl_primitive_desc_create(
-            &rpd, &rd, dnnl_attr, engine_tgt, NULL);
+    dnnl_status_t init_status
+            = dnnl_primitive_desc_create(&rpd, &rd, dnnl_attr, engine, NULL);
     dnnl_primitive_attr_destroy(dnnl_attr);
     if (init_status == dnnl_unimplemented) return r->state = UNIMPLEMENTED, OK;
     SAFE(init_status, WARN);
 
     // Return if pd is not the one being tested
-    if (is_fwd != (p.prop == dnnl_forward)) return OK;
+    if ((dir & FLAG_FWD) && (p.prop == dnnl_backward)) return OK;
 
-    const char *impl_str = query_impl_info(rpd);
-    BENCHDNN_PRINT(5, "oneDNN implementation: %s\n", impl_str);
+    r->impl_name = query_impl_info(rpd);
+    BENCHDNN_PRINT(5, "oneDNN implementation: %s\n", r->impl_name.c_str());
 
     return OK;
 }
 
+void check_known_skipped_case(const prb_t &p, res_t *r) {
+    check_known_skipped_case_common({p.cfg[SRC_LAYER].dt}, r);
+    if (r->state == SKIPPED) return;
+
+    if (p.maybe_skip()) {
+        r->state = SKIPPED, r->reason = CASE_NOT_SUPPORTED;
+        return;
+    }
+}
+
 int doit(const prb_t &p, res_t *r) {
     if (bench_mode == LIST) return r->state = LISTED, OK;
 
-    // TODO: remove early exit when int8 weights reorder supports non
-    // trivial strides
-    if (p.is_int8() && !p.trivial_strides) return r->state = SKIPPED, OK;
+    check_known_skipped_case(p, r);
+    if (r->state == SKIPPED) return OK;
 
-    dnnl_primitive_desc_t rpd;
-    SAFE(init_pd(p, rpd, r, true), WARN);
+    dnnl_primitive_t c {};
+    SAFE(init_prim(&c, init_pd, &p, r), WARN);
     if (r->state == SKIPPED || r->state == UNIMPLEMENTED) return OK;
-
-    dnnl_primitive_t c;
     auto cleanup = [&]() {
         DNN_SAFE(dnnl_primitive_destroy(c), CRIT);
         return OK;
     };
 
-    DNN_SAFE(dnnl_primitive_create(&c, rpd), WARN);
-    DNN_SAFE(dnnl_primitive_desc_destroy(rpd), CRIT);
-
     const_dnnl_primitive_desc_t const_fpd;
     DNN_SAFE(dnnl_primitive_get_primitive_desc(c, &const_fpd), CRIT);
 
     if (dnn_mem_t::check_mem_size(const_fpd) != OK) {
         DNN_SAFE(dnnl_primitive_destroy(c), CRIT);
-        return r->state = SKIPPED, OK;
+        return r->state = SKIPPED, r->reason = NOT_ENOUGH_RAM, OK;
     }
 
     const auto q = [](const_dnnl_primitive_desc_t pd,
@@ -627,34 +643,36 @@ int doit(const prb_t &p, res_t *r) {
     const auto &workspace_md = q(const_fpd, DNNL_ARG_WORKSPACE);
     const auto &scratchpad_md = q(const_fpd, DNNL_ARG_SCRATCHPAD);
 
-    dnn_mem_t src_layer_dt(src_layer_md, engine_tgt);
-    dnn_mem_t src_iter_dt(src_iter_md, engine_tgt);
-    dnn_mem_t src_iter_c_dt(src_iter_c_md, engine_tgt);
-    dnn_mem_t weights_layer_dt(weights_layer_md, engine_tgt);
-    dnn_mem_t weights_iter_dt(weights_iter_md, engine_tgt);
-    dnn_mem_t weights_peephole_dt(weights_peephole_md, engine_tgt);
-    dnn_mem_t weights_projection_dt(weights_projection_md, engine_tgt);
-    dnn_mem_t bias_dt(bias_md, engine_tgt);
-    dnn_mem_t dst_layer_dt(dst_layer_md, engine_tgt);
-    dnn_mem_t dst_iter_dt(dst_iter_md, engine_tgt);
-    dnn_mem_t dst_iter_c_dt(dst_iter_c_md, engine_tgt);
-    dnn_mem_t workspace_dt(workspace_md, engine_tgt);
-    dnn_mem_t scratchpad_dt(scratchpad_md, engine_tgt);
+    const auto &test_engine = get_test_engine();
+
+    dnn_mem_t src_layer_dt(src_layer_md, test_engine);
+    dnn_mem_t src_iter_dt(src_iter_md, test_engine);
+    dnn_mem_t src_iter_c_dt(src_iter_c_md, test_engine);
+    dnn_mem_t weights_layer_dt(weights_layer_md, test_engine);
+    dnn_mem_t weights_iter_dt(weights_iter_md, test_engine);
+    dnn_mem_t weights_peephole_dt(weights_peephole_md, test_engine);
+    dnn_mem_t weights_projection_dt(weights_projection_md, test_engine);
+    dnn_mem_t bias_dt(bias_md, test_engine);
+    dnn_mem_t dst_layer_dt(dst_layer_md, test_engine);
+    dnn_mem_t dst_iter_dt(dst_iter_md, test_engine);
+    dnn_mem_t dst_iter_c_dt(dst_iter_c_md, test_engine);
+    dnn_mem_t workspace_dt(workspace_md, test_engine);
+    dnn_mem_t scratchpad_dt(scratchpad_md, test_engine);
 
     const auto fp = dnnl_f32;
-    dnn_mem_t src_layer_fp(src_layer_md, fp, dnnl_tnc, engine_tgt);
-    dnn_mem_t src_iter_fp(src_iter_md, fp, dnnl_ldnc, engine_tgt);
-    dnn_mem_t src_iter_c_fp(src_iter_c_md, fp, dnnl_ldnc, engine_tgt);
-    dnn_mem_t weights_layer_fp(weights_layer_md, fp, dnnl_ldigo, engine_tgt);
-    dnn_mem_t weights_iter_fp(weights_iter_md, fp, dnnl_ldigo, engine_tgt);
+    dnn_mem_t src_layer_fp(src_layer_md, fp, dnnl_tnc, test_engine);
+    dnn_mem_t src_iter_fp(src_iter_md, fp, dnnl_ldnc, test_engine);
+    dnn_mem_t src_iter_c_fp(src_iter_c_md, fp, dnnl_ldnc, test_engine);
+    dnn_mem_t weights_layer_fp(weights_layer_md, fp, dnnl_ldigo, test_engine);
+    dnn_mem_t weights_iter_fp(weights_iter_md, fp, dnnl_ldigo, test_engine);
     dnn_mem_t weights_peephole_fp(
-            weights_peephole_md, fp, dnnl_ldgo, engine_tgt);
+            weights_peephole_md, fp, dnnl_ldgo, test_engine);
     dnn_mem_t weights_projection_fp(
-            weights_projection_md, fp, dnnl_ldio, engine_tgt);
-    dnn_mem_t bias_fp(bias_md, fp, dnnl_ldgo, engine_tgt);
-    dnn_mem_t dst_layer_fp(dst_layer_md, fp, dnnl_tnc, engine_tgt);
-    dnn_mem_t dst_iter_fp(dst_iter_md, fp, dnnl_ldnc, engine_tgt);
-    dnn_mem_t dst_iter_c_fp(dst_iter_c_md, fp, dnnl_ldnc, engine_tgt);
+            weights_projection_md, fp, dnnl_ldio, test_engine);
+    dnn_mem_t bias_fp(bias_md, fp, dnnl_ldgo, test_engine);
+    dnn_mem_t dst_layer_fp(dst_layer_md, fp, dnnl_tnc, test_engine);
+    dnn_mem_t dst_iter_fp(dst_iter_md, fp, dnnl_ldnc, test_engine);
+    dnn_mem_t dst_iter_c_fp(dst_iter_c_md, fp, dnnl_ldnc, test_engine);
 
     dnn_mem_t bwd_weights_layer_dt;
     dnn_mem_t bwd_weights_iter_dt;
@@ -716,7 +734,7 @@ int doit(const prb_t &p, res_t *r) {
     args.set(DNNL_ARG_WORKSPACE, workspace_dt);
     args.set(DNNL_ARG_SCRATCHPAD, scratchpad_dt);
 
-    DNN_SAFE_CLEAN(execute_and_wait(c, stream_tgt, args), WARN, cleanup);
+    SAFE_CLEAN(execute_and_wait(c, args), WARN, cleanup);
 
     if ((p.prop == dnnl_forward) && (bench_mode & CORR)) {
         compute_ref_fwd(p, src_layer_fp, src_iter_fp, src_iter_c_fp,
@@ -733,19 +751,19 @@ int doit(const prb_t &p, res_t *r) {
     }
 
     if (p.prop == dnnl_backward) {
-        SAFE(init_pd(p, rpd, r, false), WARN);
-        DNN_SAFE(dnnl_primitive_destroy(c), CRIT);
+        dnnl_primitive_t bwd_p {};
+        int status = init_prim(&bwd_p, init_pd, &p, r, FLAG_BWD);
+        dnnl_primitive_destroy(c);
+        if (status != OK) return status;
         if (r->state == SKIPPED || r->state == UNIMPLEMENTED) return OK;
-
-        DNN_SAFE(dnnl_primitive_create(&c, rpd), WARN);
-        DNN_SAFE(dnnl_primitive_desc_destroy(rpd), CRIT);
+        c = bwd_p;
 
         const_dnnl_primitive_desc_t const_bpd;
         DNN_SAFE(dnnl_primitive_get_primitive_desc(c, &const_bpd), CRIT);
 
         if (dnn_mem_t::check_mem_size(const_bpd) != OK) {
             DNN_SAFE(dnnl_primitive_destroy(c), CRIT);
-            return r->state = SKIPPED, OK;
+            return r->state = SKIPPED, r->reason = NOT_ENOUGH_RAM, OK;
         }
 
         const auto &bwd_weights_layer_md = q(const_bpd, DNNL_ARG_WEIGHTS_LAYER);
@@ -769,44 +787,46 @@ int doit(const prb_t &p, res_t *r) {
         const auto &diff_dst_iter_c_md = q(const_bpd, DNNL_ARG_DIFF_DST_ITER_C);
         const auto &bwd_scratchpad_md = q(const_bpd, DNNL_ARG_SCRATCHPAD);
 
-        bwd_weights_layer_dt = dnn_mem_t(bwd_weights_layer_md, engine_tgt);
-        bwd_weights_iter_dt = dnn_mem_t(bwd_weights_iter_md, engine_tgt);
+        bwd_weights_layer_dt = dnn_mem_t(bwd_weights_layer_md, test_engine);
+        bwd_weights_iter_dt = dnn_mem_t(bwd_weights_iter_md, test_engine);
         bwd_weights_projection_dt
-                = dnn_mem_t(bwd_weights_projection_md, engine_tgt);
-        diff_src_layer_dt = dnn_mem_t(diff_src_layer_md, engine_tgt);
-        diff_src_iter_dt = dnn_mem_t(diff_src_iter_md, engine_tgt);
-        diff_src_iter_c_dt = dnn_mem_t(diff_src_iter_c_md, engine_tgt);
-        diff_weights_layer_dt = dnn_mem_t(diff_weights_layer_md, engine_tgt);
-        diff_weights_iter_dt = dnn_mem_t(diff_weights_iter_md, engine_tgt);
+                = dnn_mem_t(bwd_weights_projection_md, test_engine);
+        diff_src_layer_dt = dnn_mem_t(diff_src_layer_md, test_engine);
+        diff_src_iter_dt = dnn_mem_t(diff_src_iter_md, test_engine);
+        diff_src_iter_c_dt = dnn_mem_t(diff_src_iter_c_md, test_engine);
+        diff_weights_layer_dt = dnn_mem_t(diff_weights_layer_md, test_engine);
+        diff_weights_iter_dt = dnn_mem_t(diff_weights_iter_md, test_engine);
         diff_weights_peephole_dt
-                = dnn_mem_t(diff_weights_peephole_md, engine_tgt);
+                = dnn_mem_t(diff_weights_peephole_md, test_engine);
         diff_weights_projection_dt
-                = dnn_mem_t(diff_weights_projection_md, engine_tgt);
-        diff_bias_dt = dnn_mem_t(diff_bias_md, engine_tgt);
-        diff_dst_layer_dt = dnn_mem_t(diff_dst_layer_md, engine_tgt);
-        diff_dst_iter_dt = dnn_mem_t(diff_dst_iter_md, engine_tgt);
-        diff_dst_iter_c_dt = dnn_mem_t(diff_dst_iter_c_md, engine_tgt);
-        scratchpad_dt = dnn_mem_t(bwd_scratchpad_md, engine_tgt);
+                = dnn_mem_t(diff_weights_projection_md, test_engine);
+        diff_bias_dt = dnn_mem_t(diff_bias_md, test_engine);
+        diff_dst_layer_dt = dnn_mem_t(diff_dst_layer_md, test_engine);
+        diff_dst_iter_dt = dnn_mem_t(diff_dst_iter_md, test_engine);
+        diff_dst_iter_c_dt = dnn_mem_t(diff_dst_iter_c_md, test_engine);
+        scratchpad_dt = dnn_mem_t(bwd_scratchpad_md, test_engine);
 
         dnn_mem_t diff_src_layer_fp(
-                diff_src_layer_md, fp, dnnl_tnc, engine_tgt);
-        dnn_mem_t diff_src_iter_fp(diff_src_iter_md, fp, dnnl_ldnc, engine_tgt);
+                diff_src_layer_md, fp, dnnl_tnc, test_engine);
+        dnn_mem_t diff_src_iter_fp(
+                diff_src_iter_md, fp, dnnl_ldnc, test_engine);
         dnn_mem_t diff_src_iter_c_fp(
-                diff_src_iter_c_md, fp, dnnl_ldnc, engine_tgt);
+                diff_src_iter_c_md, fp, dnnl_ldnc, test_engine);
         dnn_mem_t diff_weights_layer_fp(
-                diff_weights_layer_md, fp, dnnl_ldigo, engine_tgt);
+                diff_weights_layer_md, fp, dnnl_ldigo, test_engine);
         dnn_mem_t diff_weights_iter_fp(
-                diff_weights_iter_md, fp, dnnl_ldigo, engine_tgt);
+                diff_weights_iter_md, fp, dnnl_ldigo, test_engine);
         dnn_mem_t diff_weights_peephole_fp(
-                diff_weights_peephole_md, fp, dnnl_ldgo, engine_tgt);
+                diff_weights_peephole_md, fp, dnnl_ldgo, test_engine);
         dnn_mem_t diff_weights_projection_fp(
-                diff_weights_projection_md, fp, dnnl_ldio, engine_tgt);
-        dnn_mem_t diff_bias_fp(diff_bias_md, fp, dnnl_ldgo, engine_tgt);
+                diff_weights_projection_md, fp, dnnl_ldio, test_engine);
+        dnn_mem_t diff_bias_fp(diff_bias_md, fp, dnnl_ldgo, test_engine);
         dnn_mem_t diff_dst_layer_fp(
-                diff_dst_layer_md, fp, dnnl_tnc, engine_tgt);
-        dnn_mem_t diff_dst_iter_fp(diff_dst_iter_md, fp, dnnl_ldnc, engine_tgt);
+                diff_dst_layer_md, fp, dnnl_tnc, test_engine);
+        dnn_mem_t diff_dst_iter_fp(
+                diff_dst_iter_md, fp, dnnl_ldnc, test_engine);
         dnn_mem_t diff_dst_iter_c_fp(
-                diff_dst_iter_c_md, fp, dnnl_ldnc, engine_tgt);
+                diff_dst_iter_c_md, fp, dnnl_ldnc, test_engine);
 
         SAFE(bwd_weights_iter_dt.reorder(weights_iter_dt), WARN);
         SAFE(bwd_weights_layer_dt.reorder(weights_layer_dt), WARN);
@@ -873,7 +893,7 @@ int doit(const prb_t &p, res_t *r) {
         args.set(DNNL_ARG_DIFF_BIAS, diff_bias_dt);
         args.set(DNNL_ARG_SCRATCHPAD, scratchpad_dt);
 
-        DNN_SAFE_CLEAN(execute_and_wait(c, stream_tgt, args), WARN, cleanup);
+        SAFE_CLEAN(execute_and_wait(c, args), WARN, cleanup);
 
         if (bench_mode & CORR) {
             compute_ref_bwd(p, src_layer_fp, src_iter_fp, src_iter_c_fp,
