@@ -17,7 +17,6 @@
 #include "dnnl_test_common.hpp"
 #include "gtest/gtest.h"
 
-#include "cpu_isa_traits.hpp"
 #include "dnnl.hpp"
 
 #include <vector>
@@ -93,13 +92,8 @@ protected:
         matmul_test_params p
                 = ::testing::TestWithParam<decltype(p)>::GetParam();
 
-        SKIP_IF(p.base.src.dt == memory::data_type::f16
-                        && get_test_engine_kind() == engine::kind::cpu,
-                "CPU does not support f16 data type.");
-        SKIP_IF(p.base.src.dt == memory::data_type::bf16
-                        && get_test_engine_kind() == engine::kind::cpu
-                        && !impl::cpu::mayiuse(impl::cpu::avx512_core),
-                "current ISA doesn't support bfloat16 data type");
+        SKIP_IF(unsupported_data_type(p.base.src.dt),
+                "Engine does not support this data type.");
 
         catch_expected_failures(
                 [=]() { Test(); }, p.expect_to_fail, p.expected_status, false);
