@@ -234,6 +234,10 @@ protected:
         test_resampling_desc_t pd = p.test_pd;
 
         memory::dims src_dims = {pd.mb, pd.c}, dst_dims = {pd.mb, pd.c};
+        // When `out_of_memory` testing is enabled, factors is expanded each
+        // time `Test()` is executed for any test. Clear the vector to avoid
+        // having its size > DNNL_MAX_NDIMS.
+        factors.clear();
         if (p.ndims == 5) {
             factors.push_back(pd.fd);
             src_dims.push_back(pd.id);
@@ -334,8 +338,13 @@ using resampling_test_float = resampling_test<float>;
 
 TEST_P(resampling_test_float, TestsResampleF32) {}
 
-CPU_INSTANTIATE_TEST_SUITE_P(TestResampleForwardPlainLinear,
-        resampling_test_float,
+INSTANTIATE_TEST_SUITE_P(TestResampleEF, resampling_test_float,
+        ::testing::Values(resampling_test_params {prop_kind::forward,
+                algorithm::resampling_linear, memory::format_tag::any,
+                EXPAND_SIZES_1D(1, 1, 5, 10, 2.f), true,
+                dnnl_invalid_arguments}));
+
+INSTANTIATE_TEST_SUITE_P(TestResampleForwardPlainLinear, resampling_test_float,
         ::testing::Values(
                 resampling_test_params {prop_kind::forward,
                         algorithm::resampling_linear, memory::format_tag::ncw,
@@ -360,7 +369,7 @@ CPU_INSTANTIATE_TEST_SUITE_P(TestResampleForwardPlainLinear,
                         EXPAND_SIZES_3D(
                                 1, 16, 5, 10, 1, 10, 5, 1, 2.f, 0.5f, 1.f)}));
 
-CPU_INSTANTIATE_TEST_SUITE_P(TestResampleForwardBlockedLinear,
+INSTANTIATE_TEST_SUITE_P(TestResampleForwardBlockedLinear,
         resampling_test_float,
         ::testing::Values(
                 resampling_test_params {prop_kind::forward,
@@ -376,7 +385,7 @@ CPU_INSTANTIATE_TEST_SUITE_P(TestResampleForwardBlockedLinear,
                         EXPAND_SIZES_3D(
                                 1, 1, 5, 10, 15, 10, 5, 7, 2.f, 0.5f, 0.5f)}));
 
-CPU_INSTANTIATE_TEST_SUITE_P(TestResampleForwardPlainNN, resampling_test_float,
+INSTANTIATE_TEST_SUITE_P(TestResampleForwardPlainNN, resampling_test_float,
         ::testing::Values(
                 resampling_test_params {prop_kind::forward,
                         algorithm::resampling_nearest, memory::format_tag::ncw,
@@ -399,8 +408,7 @@ CPU_INSTANTIATE_TEST_SUITE_P(TestResampleForwardPlainNN, resampling_test_float,
                         EXPAND_SIZES_3D(
                                 5, 5, 5, 10, 15, 10, 5, 7, 2.f, 0.5f, 0.5f)}));
 
-CPU_INSTANTIATE_TEST_SUITE_P(TestResampleForwardBlockedNN,
-        resampling_test_float,
+INSTANTIATE_TEST_SUITE_P(TestResampleForwardBlockedNN, resampling_test_float,
         ::testing::Values(
                 resampling_test_params {prop_kind::forward,
                         algorithm::resampling_nearest,
