@@ -17,7 +17,6 @@
 #include "dnnl_test_common.hpp"
 #include "gtest/gtest.h"
 
-#include "cpu_isa_traits.hpp"
 #include "dnnl.hpp"
 
 namespace dnnl {
@@ -140,9 +139,8 @@ protected:
         SKIP_IF(get_test_engine_kind() == engine::kind::gpu
                         && src_data_type == memory::data_type::bf16,
                 "GPU does not support bfloat16 data type.");
-        SKIP_IF(src_data_type == memory::data_type::bf16
-                        && !impl::cpu::mayiuse(impl::cpu::avx512_core),
-                "current ISA doesn't support bfloat16 data type");
+        SKIP_IF(unsupported_data_type(src_data_type),
+                "Engine does not support this data type.");
         catch_expected_failures(
                 [=]() { Test(); }, p.expect_to_fail, p.expected_status);
     }
@@ -196,13 +194,13 @@ protected:
 
             ASSERT_EQ(sum_pd.dst_desc().data.ndims, dst_desc.data.ndims);
         }
-        ASSERT_NO_THROW(dst = memory(sum_pd.dst_desc(), eng));
+        dst = memory(sum_pd.dst_desc(), eng);
         // test construction from a C pd
         sum_pd = sum::primitive_desc(sum_pd.get());
 
         ASSERT_TRUE(sum_pd.query_md(query::exec_arg_md, DNNL_ARG_DST)
                 == sum_pd.dst_desc());
-        for (size_t i = 0; i < srcs.size(); i++)
+        for (int i = 0; i < (int)srcs.size(); i++)
             ASSERT_TRUE(sum_pd.query_md(
                                 query::exec_arg_md, DNNL_ARG_MULTIPLE_SRC + i)
                     == sum_pd.src_desc(i));
