@@ -43,10 +43,7 @@
 
 #include "mkldnn.hpp"
 
-// MSVC doesn't support collapse clause in omp parallel
-#if defined(_MSC_VER) && !defined(__clang__) && !defined(__INTEL_COMPILER)
-#define collapse(x)
-#endif
+#include "example_utils.hpp"
 
 using namespace mkldnn;
 
@@ -85,9 +82,7 @@ void compute_weighted_annotations(float *weighted_annotations,
 
 void compute_sum_of_rows(
         int8_t *a, dim_t rows, dim_t cols, int32_t *a_reduced) {
-#ifdef _OPENMP
-#pragma omp parallel for
-#endif
+    PRAGMA_OMP_PARALLEL_FOR_COLLAPSE(1)
     for (dim_t i = 0; i < cols; i++) {
         a_reduced[i] = 0;
         for (dim_t j = 0; j < rows; j++) {
@@ -120,9 +115,7 @@ void compute_attention(float *context_vectors, dim_t src_seq_length_max,
 
     // then we compute the alignment model
     float *alignment_model_ptr = alignment_model.data();
-#ifdef _OPENMP
-#pragma omp parallel for collapse(2)
-#endif
+    PRAGMA_OMP_PARALLEL_FOR_COLLAPSE(2)
     for (dim_t i = 0; i < src_seq_length_max; i++) {
         for (dim_t j = 0; j < batch; j++) {
             for (dim_t k = 0; k < feature_size; k++) {
@@ -144,15 +137,11 @@ void compute_attention(float *context_vectors, dim_t src_seq_length_max,
             alignment_model_ptr, feature_size, weights_alignments, 1, 0.f,
             alignments.data(), 1);
 
-// softmax on alignments. the resulting context weights are in alignments
-#ifdef _OPENMP
-#pragma omp parallel for
-#endif
+    // softmax on alignments. the resulting context weights are in alignments
+    PRAGMA_OMP_PARALLEL_FOR_COLLAPSE(1)
     for (dim_t i = 0; i < batch; i++)
         exp_sums[i] = 0.0f;
-#ifdef _OPENMP
-#pragma omp parallel for collapse(2)
-#endif
+    PRAGMA_OMP_PARALLEL_FOR_COLLAPSE(2)
     for (dim_t i = 0; i < src_seq_length_max; i++) {
         for (dim_t j = 0; j < batch; j++) {
             alignments[i * batch + j] = expf(alignments[i * batch + j]);
@@ -160,26 +149,20 @@ void compute_attention(float *context_vectors, dim_t src_seq_length_max,
         }
     }
 
-#ifdef _OPENMP
-#pragma omp parallel for collapse(2)
-#endif
+    PRAGMA_OMP_PARALLEL_FOR_COLLAPSE(2)
     for (dim_t i = 0; i < src_seq_length_max; i++)
         for (dim_t j = 0; j < batch; j++)
             alignments[i * batch + j] /= exp_sums[j];
 
-// then we compute the context vectors
-#ifdef _OPENMP
-#pragma omp parallel for collapse(2)
-#endif
+    // then we compute the context vectors
+    PRAGMA_OMP_PARALLEL_FOR_COLLAPSE(2)
     for (dim_t i = 0; i < batch; i++)
         for (dim_t j = 0; j < feature_size; j++)
             context_vectors[i * (feature_size + feature_size) + feature_size
                     + j]
                     = 0.0f;
 
-#ifdef _OPENMP
-#pragma omp parallel for collapse(3)
-#endif
+    PRAGMA_OMP_PARALLEL_FOR_COLLAPSE(3)
     for (dim_t i = 0; i < batch; i++)
         for (dim_t k = 0; k < src_seq_length_max; k++)
             for (dim_t j = 0; j < feature_size; j++)
@@ -194,10 +177,8 @@ void compute_attention(float *context_vectors, dim_t src_seq_length_max,
 
 void copy_context(
         float *src_iter, dim_t n_layers, dim_t batch, dim_t feature_size) {
-// we copy the context from the first layer to all other layers
-#ifdef _OPENMP
-#pragma omp parallel for collapse(3)
-#endif
+    // we copy the context from the first layer to all other layers
+    PRAGMA_OMP_PARALLEL_FOR_COLLAPSE(3)
     for (dim_t k = 1; k < n_layers; k++)
         for (dim_t j = 0; j < batch; j++)
             for (dim_t i = 0; i < feature_size; i++)
@@ -368,6 +349,19 @@ void simple_net() {
     attr.set_rnn_data_qparams(data_scale, data_shift);
     attr.set_rnn_weights_qparams(weights_scale_mask, weights_scales);
 
+    try {
+        auto enc_bidir_prim_desc
+                = lstm_forward::primitive_desc(bi_layer_desc, attr, cpu_engine);
+    } catch (error &e) {
+        if (e.status == mkldnn_unimplemented) {
+            std::cerr << "Intel MKL-DNN does not have int8 RNN "
+                         "implementation that supports this system. Please "
+                         "refer to the developer guide for details."
+                      << std::endl;
+        }
+        throw;
+    }
+
     auto enc_bidir_prim_desc
             = lstm_forward::primitive_desc(bi_layer_desc, attr, cpu_engine);
     //[RNN attri]
