@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 
 ## Copyright 2018-2020 Intel Corporation
 ## SPDX-License-Identifier: Apache-2.0
@@ -20,16 +20,22 @@ def main():
   device = init_device(cfg)
 
   # Load the checkpoint
+  print('Result:', cfg.result)
   checkpoint = load_checkpoint(cfg, device, cfg.checkpoint)
   epoch = checkpoint['epoch']
   model_state = checkpoint['model_state']
+  print('Epoch:', epoch)
 
   # Save the weights to a TZA file
-  output_filename = os.path.join(get_result_dir(cfg), cfg.result)
-  if cfg.checkpoint:
-    output_filename += '_%d' % epoch
-  output_filename += '.tza'
-  print('Saving weights:', output_filename)
+  if cfg.output:
+    output_filename = cfg.output
+  else:
+    output_filename = os.path.join(get_result_dir(cfg), cfg.result)
+    if cfg.checkpoint:
+      output_filename += '_%d' % epoch
+    output_filename += '.tza'
+  print('Output:', output_filename)
+  print()
 
   with tza.Writer(output_filename) as output_file:
     for name, value in model_state.items():
